#include <nexus-ldap-realm-plugin.h>

/**
 * Enunciate-specific C functions.
 */
#ifndef ENUNCIATE_C_UTILITIES
#define ENUNCIATE_C_UTILITIES

/*******************xml utilities************************************/

static int xmlTextReaderAdvanceToNextStartOrEndElement(xmlTextReaderPtr reader) {
  int status = xmlTextReaderRead(reader);
  while (status && xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT && xmlTextReaderNodeType(reader) != XML_READER_TYPE_END_ELEMENT) {
    status = xmlTextReaderRead(reader);
  }
  return status;
}

static int xmlTextReaderSkipElement(xmlTextReaderPtr reader) {
  int status = xmlTextReaderNext(reader);
  while (status && xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT && xmlTextReaderNodeType(reader) != XML_READER_TYPE_END_ELEMENT) {
    status = xmlTextReaderRead(reader);
  }
  return status;
}

static xmlChar *xmlTextReaderReadEntireNodeValue(xmlTextReaderPtr reader) {
  xmlChar *buffer = calloc(1, sizeof(xmlChar));
  const xmlChar *snippet;
  int status;
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ATTRIBUTE) {
    return xmlTextReaderValue(reader);
  }
  else if (xmlTextReaderIsEmptyElement(reader) == 0) {
    status = xmlTextReaderRead(reader);
    while (status && (xmlTextReaderNodeType(reader) == XML_READER_TYPE_TEXT || xmlTextReaderNodeType(reader) == XML_READER_TYPE_CDATA || xmlTextReaderNodeType(reader) == XML_READER_TYPE_ENTITY_REFERENCE)) {
      snippet = xmlTextReaderConstValue(reader);
      buffer = realloc(buffer, (xmlStrlen(buffer) + xmlStrlen(snippet) + 1) * sizeof(xmlChar));
      xmlStrcat(buffer, snippet);
      status = xmlTextReaderRead(reader);
    }
  }
  return buffer;
}

/*******************base 64 utilities************************************/

/*
 * Base64 Translation Table as described in RFC1113.
 *
 * This code was graciously ripped from http://base64.sourceforge.net
 */
static const char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

/*
 * Base64 Translation Table to decode (created by author)
 *
 * This code was graciously ripped from http://base64.sourceforge.net
 */
static const char cd64[]="|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq";

/*
 * encode 3 8-bit binary bytes as 4 '6-bit' characters
 *
 * This code was graciously ripped from http://base64.sourceforge.net
 *
 * @param in the block to encode
 * @param out the block to encode to
 * @param len the length of the 'in' block.
 */
static void _encode_base64_block(unsigned char in[3], unsigned char out[4], int len) {
  out[0] = cb64[ in[0] >> 2 ];
  out[1] = cb64[ ((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4) ];
  out[2] = (unsigned char) (len > 1 ? cb64[ ((in[1] & 0x0f) << 2) | ((in[2] & 0xc0) >> 6) ] : '=');
  out[3] = (unsigned char) (len > 2 ? cb64[ in[2] & 0x3f ] : '=');
}

/*
 * decode 4 '6-bit' characters into 3 8-bit binary bytes
 *
 * This code was graciously ripped from http://base64.sourceforge.net
 */
static void _decode_base64_block( unsigned char in[4], unsigned char out[3] )
{
    out[ 0 ] = (unsigned char ) (in[0] << 2 | in[1] >> 4);
    out[ 1 ] = (unsigned char ) (in[1] << 4 | in[2] >> 2);
    out[ 2 ] = (unsigned char ) (((in[2] << 6) & 0xc0) | in[3]);
}

/*
 * base64 encode a stream adding padding and line breaks as per spec.
 *
 * This code was graciously ripped from http://base64.sourceforge.net
 *
 * @param instream The stream to encode.
 * @param insize The size of the stream to encode.
 * @return The encoded string.
 */
xmlChar *_encode_base64(unsigned char *instream, int insize) {
  unsigned char in[3];
  unsigned char out[4];
  xmlChar *encoded;
  int i, in_index = 0, out_index = 0, blocklen;

  if (insize == 0) {
    return BAD_CAST "\0";
  }

  encoded = calloc(((insize / 3) * 4) + 10, sizeof(xmlChar));
  while (in_index <= insize) {
    blocklen = 0;
    for (i = 0; i < 3; i++) {
      in[i] = instream[in_index++];
      if (in_index <= insize) {
        blocklen++;
      }
      else {
        in[i] = 0;
      }
    }
    if (blocklen) {
      _encode_base64_block(in, out, blocklen);
      for( i = 0; i < 4; i++ ) {
        encoded[out_index++] = out[i];
      }
    }
  }

  return encoded;
}

/*
 * Decode a base64 encoded stream discarding padding, line breaks and noise
 *
 * This code was graciously ripped from http://base64.sourceforge.net
 *
 * @param invalue The string to decode.
 * @param outsize Holder for the length of the returned data.
 * @return The decoded data.
 */
unsigned char *_decode_base64( const xmlChar *invalue, int *outsize ) {
  xmlChar in[4];
  unsigned char out[3], v;
  int i, in_index = 0, out_index = 0, blocklen;
  unsigned char *outstream;

  if (invalue == NULL) {
    return NULL;
  }

  outstream = calloc(((xmlStrlen(invalue) / 4) * 3) + 1, sizeof(unsigned char));
  while (invalue[in_index] != '\0') {
    for (blocklen = 0, i = 0; i < 4 && invalue[in_index]; i++) {
      v = 0;
      while (invalue[in_index] != '\0' && v == 0) {
        v = (unsigned char) invalue[in_index++];
        v = (unsigned char) ((v < 43 || v > 122) ? 0 : cd64[ v - 43 ]);
        if (v) {
          v = (unsigned char) ((v == '$') ? 0 : v - 61);
        }
      }

      if (invalue[in_index] != '\0') {
        blocklen++;
        if (v) {
          in[i] = (unsigned char) (v - 1);
        }
      }
      else {
        in[i] = 0;
      }
    }

    if (blocklen) {
      _decode_base64_block( in, out );
      for( i = 0; i < blocklen - 1; i++ ) {
        outstream[out_index++] = out[i];
      }
    }
  }

  if (outsize != NULL) {
    *outsize = out_index;
  }

  return outstream;
}

#endif /* ENUNCIATE_C_UTILITIES */

#ifndef ENUNCIATE_OBJC_CLASSES
#define ENUNCIATE_OBJC_CLASSES

/**
 * Protocol defining a JAXB (see https://jaxb.dev.java.net/) type.
 */
@protocol JAXBType

/**
 * Read an XML type from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of the object defining the JAXB type.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader;

/**
 * Initialize the object with an XML reader.
 *
 * @param reader The XML reader from which to initialize the values of this type.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader;

/**
 * Write this instance of a JAXB type to a writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer;

@end /*protocol JAXBType*/


/**
 * Protocol defining a JAXB (see https://jaxb.dev.java.net/) element.
 */
@protocol JAXBElement

/**
 * Read the XML element from an XML reader. It is assumed
 * that the reader is pointing at the start element (be careful
 * that it's not still pointing to the XML document).
 *
 * @param reader The reader.
 * @return An instance of the object defining the JAXB element.
 */
+ (id<JAXBElement>) readXMLElement: (xmlTextReaderPtr) reader;

/**
 * Write this instance of a JAXB element to a writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer;

/**
 * Write this instance of a JAXB element to a writer.
 *
 * @param writer The writer.
 * @param writeNs Whether to write the namespaces for this element to the xml writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer writeNamespaces: (BOOL) writeNs;

@end /*protocol JAXBElement*/


/**
 * Protocol defining methods for events that occur
 * when reading/parsing XML. Intended for internal
 * use only.
 */
@protocol JAXBReading

/**
 * Method for reading an attribute.
 *
 * @param reader The reader pointing to the attribute.
 * @return Whether the attribute was read.
 */
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader;

/**
 * Method for reading the value of an element.
 *
 * @param reader The reader pointing to the element containing a value.
 * @return Whether the value was read.
 */
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader;

/**
 * Method for reading a child element. If (and only if) the child
 * element was handled, the element in the reader should be
 * "consumed" and the reader will be pointing to the end element.
 *
 * @param reader The reader pointing to the child element.
 * @return Whether the child element was read.
 */
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader;

/**
 * Method for reading an unknown attribute.
 *
 * @param reader The reader pointing to the unknown attribute.
 * @return Whether the attribute was read.
 */
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader;

/**
 * Method for reading an unknown child element. Implementations
 * must be responsible for "consuming" the child element.
 *
 * @param reader The reader pointing to the unknown child element.
 * @return The status of the reader after having consumed the unknown child element.
 */
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader;

@end /*protocol JAXBReading*/

/**
 * Protocol defining methods for events that occur
 * when writing XML. Intended for internal
 * use only.
 */
@protocol JAXBWriting

/**
 * Method for writing the attributes.
 *
 * @param writer The writer.
 */
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer;

/**
 * Method for writing the element value.
 *
 * @param writer The writer.
 */
- (void) writeJAXBValue: (xmlTextWriterPtr) writer;

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer;

@end /*protocol JAXBWriting*/

/**
 * Declaration of the JAXB type, element, events for a base object.
 */
@interface NSObject (JAXB) <JAXBType, JAXBElement, JAXBReading, JAXBWriting>

@end

/**
 * Implementation of the JAXB type, element, events for a base object.
 */
@implementation NSObject (JAXB)

/**
 * Read the XML type from the reader; an instance of NSXMLElement.
 *
 * @param reader The reader.
 * @return An instance of NSXMLElement
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  return [JAXBBasicXMLNode readXMLType: reader];
}

/**
 * Read an XML type from an XML reader into an existing instance.
 *
 * @param reader The reader.
 * @param existing The existing instance into which to read values.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  int status, depth;
  if ((self = [self init])) {
    if (xmlTextReaderHasAttributes(reader)) {
      while (xmlTextReaderMoveToNextAttribute(reader)) {
        if ([self readJAXBAttribute: reader] == NO) {
          [self readUnknownJAXBAttribute: reader];
        }
      }

      status = xmlTextReaderMoveToElement(reader);
      if (!status) {
        //panic: unable to return to the element node.
        [NSException raise: @"XMLReadError"
                     format: @"Error moving back to element position from attributes."];
      }
    }

    if ([self readJAXBValue: reader] == NO) {
      //no value handled; attempt to process child elements
      if (xmlTextReaderIsEmptyElement(reader) == 0) {
        depth = xmlTextReaderDepth(reader);//track the depth.
        status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
        while (xmlTextReaderDepth(reader) > depth) {
          if (status < 1) {
            //panic: XML read error.
            [NSException raise: @"XMLReadError"
                         format: @"Error handling a child element."];
          }
          else if ([self readJAXBChildElement: reader]) {
            status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
          }
          else {
            status = [self readUnknownJAXBChildElement: reader];
          }
        }
      }
    }
  }
  return self;
}

/**
 * Write the XML type value of this object to the writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [self writeJAXBAttributes: writer];
  [self writeJAXBValue: writer];
  [self writeJAXBChildElements: writer];
}

/**
 * Read the XML element from the reader; an instance of NSXMLElement.
 *
 * @param reader The reader.
 * @return An instance of NSXMLElement
 */
+ (id<JAXBElement>) readXMLElement: (xmlTextReaderPtr) reader
{
  return (id<JAXBElement>) [JAXBBasicXMLNode readXMLType: reader];
}

/**
 * No op; root objects don't have an element name/namespace. Subclasses must override.
 *
 * @param writer The writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer
{
  //no-op
}

/**
 * No op; root objects don't have an element name/namespace. Subclasses must override.
 *
 * @param writer The writer.
 * @param writeNs Ignored.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer writeNamespaces: (BOOL) writeNs
{
  //no-op
}

/**
 * No-op; base objects do not handle any attributes.
 *
 * @param reader The reader pointing to the attribute.
 * @return NO
 */
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  return NO;
}

/**
 * No-op; base objects do not handle any values.
 *
 * @param reader The reader pointing to the element containing a value.
 * @return NO
 */
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return NO;
}

/**
 * No-op; base objects do not handle any child elements.
 *
 * @param reader The reader pointing to the child element.
 * @return NO
 */
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  return NO;
}

/**
 * No-op; base objects do not handle any attributes.
 *
 * @param reader The reader pointing to the unknown attribute.
 */
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
}

/**
 * Just skips the unknown element; base objects do not handle any child elements.
 *
 * @param reader The reader pointing to the unknown child element.
 * @return The status of the reader after skipping the unknown child element.
 */
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return xmlTextReaderSkipElement(reader);
}

/**
 * No-op; base objects have no attributes.
 *
 * @param writer The writer.
 */
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  //no-op.
}

/**
 * No-op; base objects have no element value.
 *
 * @param writer The writer.
 */
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  //no-op.
}

/**
 * No-op; base objects have no child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  //no-op.
}

@end /*NSObject (JAXB)*/

/**
 * Implementation of the JAXB type, element for an xml element.
 */
@implementation JAXBBasicXMLNode

/**
 * Accessor for the (local) name of the XML node.
 *
 * @return The (local) name of the XML node.
 */
- (NSString *) name
{
  return _name;
}

/**
 * Accessor for the (local) name of the XML node.
 *
 * @param newName The (local) name of the XML node.
 */
- (void) setName: (NSString *) newName
{
  [newName retain];
  [_name release];
  _name = newName;
}

/**
 * Accessor for the namespace of the XML node.
 *
 * @return The namespace of the XML node.
 */
- (NSString *) ns
{
  return _ns;
}

/**
 * Accessor for the namespace of the XML node.
 *
 * @param newNs The namespace of the XML node.
 */
- (void) setNs: (NSString *) newNs
{
  [newNs retain];
  [_ns release];
  _ns = newNs;
}

/**
 * Accessor for the namespace prefix of the XML node.
 *
 * @return The namespace prefix of the XML node.
 */
- (NSString *) prefix
{
  return _prefix;
}

/**
 * Accessor for the namespace prefix of the XML node.
 *
 * @param newPrefix The namespace prefix of the XML node.
 */
- (void) setPrefix: (NSString *) newPrefix
{
  [newPrefix retain];
  [_prefix release];
  _prefix = newPrefix;
}

/**
 * Accessor for the value of the XML node.
 *
 * @return The value of the XML node.
 */
- (NSString *) value
{
  return _value;
}

/**
 * Accessor for the value of the XML node.
 *
 * @param newValue The value of the XML node.
 */
- (void) setValue: (NSString *) newValue
{
  [newValue retain];
  [_value release];
  _value = newValue;
}

/**
 * Accessor for the child elements of the XML node.
 *
 * @return The child elements of the XML node.
 */
- (NSArray *) childElements
{
  return _childElements;
}

/**
 * Accessor for the child elements of the XML node.
 *
 * @param newValue The child elements of the XML node.
 */
- (void) setChildElements: (NSArray *) newChildElements
{
  [newChildElements retain];
  [_childElements release];
  _childElements = newChildElements;
}

/**
 * Accessor for the attributes of the XML node.
 *
 * @return The attributes of the XML node.
 */
- (NSArray *) attributes
{
  return _attributes;
}

/**
 * Accessor for the attributes of the XML node.
 *
 * @param newAttributes The attributes of the XML node.
 */
- (void) setAttributes: (NSArray *) newAttributes
{
  [newAttributes retain];
  [_attributes release];
  _attributes = newAttributes;
}

- (void) dealloc {
  [self setName: nil];
  [self setNs: nil];
  [self setPrefix: nil];
  [self setValue: nil];
  [self setChildElements: nil];
  [self setAttributes: nil];
  [super dealloc];
}
@end /*implementation JAXBBasicXMLNode*/

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface JAXBBasicXMLNode (JAXB) <JAXBType, JAXBElement>

@end /*interface JAXBBasicXMLNode (JAXB)*/

@implementation JAXBBasicXMLNode (JAXB)

/**
 * Read the XML type from the reader; an instance of JAXBBasicXMLNode.
 *
 * @param reader The reader.
 * @return An instance of JAXBBasicXMLNode
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  JAXBBasicXMLNode *node = [[JAXBBasicXMLNode alloc] init];
  NS_DURING
  {
    [node initWithReader: reader];
  }
  NS_HANDLER
  {
    [node dealloc];
    node = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [node autorelease];
  return node;
}

/**
 * Read an XML type from an XML reader into an existing instance of JAXBBasicXMLNode.
 *
 * @param reader The reader.
 * @param existing The existing instance into which to read values.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  int status, depth;
  JAXBBasicXMLNode *child;
  xmlChar *value = NULL;
  const xmlChar *text;
  NSMutableArray *children;

  if ((self = [self init])) {
    depth = xmlTextReaderDepth(reader);
    [self setName: [NSString stringWithUTF8String: (const char *) xmlTextReaderLocalName(reader)]];
    [self setNs: [NSString stringWithUTF8String: (const char *) xmlTextReaderNamespaceUri(reader)]];
    [self setPrefix: [NSString stringWithUTF8String: (const char *)xmlTextReaderPrefix(reader)]];

    if (xmlTextReaderHasAttributes(reader)) {
      child = nil;
      children = [[NSMutableArray alloc] init];
      while (xmlTextReaderMoveToNextAttribute(reader)) {
        child = [[JAXBBasicXMLNode alloc] init];
        [child setName: [NSString stringWithUTF8String: (const char *) xmlTextReaderLocalName(reader)]];
        [child setNs: [NSString stringWithUTF8String: (const char *)xmlTextReaderNamespaceUri(reader)]];
        [child setPrefix: [NSString stringWithUTF8String: (const char *) xmlTextReaderPrefix(reader)]];
        [child setValue: [NSString stringWithUTF8String: (const char *) xmlTextReaderValue(reader)]];
        [children addObject: child];
      }
      [self setAttributes: children];

      status = xmlTextReaderMoveToElement(reader);
      if (status < 1) {
        //panic: unable to return to the element node.
        [NSException raise: @"XMLReadError"
                     format: @"Error moving to element from attributes."];
      }
    }

    if (xmlTextReaderIsEmptyElement(reader) == 0) {
      children = [[NSMutableArray alloc] init];
      status = xmlTextReaderRead(reader);
      while (status == 1 && xmlTextReaderDepth(reader) > depth) {
        switch (xmlTextReaderNodeType(reader)) {
          case XML_READER_TYPE_ELEMENT:
            child = (JAXBBasicXMLNode *) [JAXBBasicXMLNode readXMLType: reader];
            [children addObject: child];
            break;
          case XML_READER_TYPE_TEXT:
          case XML_READER_TYPE_CDATA:
            text = xmlTextReaderConstValue(reader);
            value = xmlStrncat(value, text, xmlStrlen(text));
            break;
          default:
            //skip anything else.
            break;
        }

        status = xmlTextReaderRead(reader);
      }

      if (status < 1) {
        //panic: xml read error
        [NSException raise: @"XMLReadError"
                     format: @"Error reading child elements."];
      }

      if ([children count] > 0) {
        [self setChildElements: children];
      }

      if (value != NULL) {
        [self setValue: [NSString stringWithUTF8String: (const char *) value]];
      }

    }
  }

  return self;
}

/**
 * Read the XML element from the reader; an instance of NSXMLElement.
 *
 * @param reader The reader.
 * @return An instance of NSXMLElement
 */
+ (id<JAXBElement>) readXMLElement: (xmlTextReaderPtr) reader
{
  return (id<JAXBElement>) [JAXBBasicXMLNode readXMLType: reader];
}

/**
 * Write the basic node to an xml writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  int status;
  NSEnumerator *enumerator;
  JAXBBasicXMLNode *child;
  xmlChar *childns, *childname, *childprefix, *childvalue;

  status = xmlTextWriterStartElementNS(writer, _prefix ? BAD_CAST [_prefix UTF8String] : NULL, _name ? BAD_CAST [_name UTF8String] : NULL, _ns ? BAD_CAST [_ns UTF8String] : NULL);
  if (status < 0) {
    childns = BAD_CAST "";
    if (_ns) {
      childns = BAD_CAST [_ns UTF8String];
    }

    childname = BAD_CAST "";
    if (_name) {
      childname = BAD_CAST [_name UTF8String];
    }

    [NSException raise: @"XMLWriteError"
                 format: @"Error writing start element {%s}%s for JAXBBasicXMLNode.", childns, childname];
  }

  if (_attributes) {
    enumerator = [_attributes objectEnumerator];
    while ( (child = (JAXBBasicXMLNode *)[enumerator nextObject]) ) {
      childns = NULL;
      if ([child ns]) {
        childns = BAD_CAST [[child ns] UTF8String];
      }

      childprefix = NULL;
      if ([child prefix]) {
        childprefix = BAD_CAST [[child prefix] UTF8String];
      }

      childname = NULL;
      if ([child name]) {
        childname = BAD_CAST [[child name] UTF8String];
      }

      childvalue = NULL;
      if ([child value]) {
        childvalue = BAD_CAST [[child value] UTF8String];
      }

      status = xmlTextWriterWriteAttributeNS(writer, childprefix, childname, childns, childvalue);
      if (status < 0) {
        [NSException raise: @"XMLWriteError"
                     format: @"Error writing attribute {%s}%s for JAXBBasicXMLNode.", childns, childname];
      }
    }
  }

  if (_value) {
    status = xmlTextWriterWriteString(writer, BAD_CAST [_value UTF8String]);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing value of JAXBBasicXMLNode."];
    }
  }

  if (_childElements) {
    enumerator = [_childElements objectEnumerator];
    while ( (child = [enumerator nextObject]) ) {
      [child writeXMLType: writer];
    }
  }

  status = xmlTextWriterEndElement(writer);
  if (status < 0) {
    childns = BAD_CAST "";
    if (_ns) {
      childns = BAD_CAST [_ns UTF8String];
    }

    childname = BAD_CAST "";
    if (_name) {
      childname = BAD_CAST [_name UTF8String];
    }

    [NSException raise: @"XMLWriteError"
                 format: @"Error writing end element {%s}%s for JAXBBasicXMLNode.", childns, childname];
  }
}

/**
 * Writes this node to a writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer
{
  [self writeXMLType: writer];
}

/**
 * Writes this node to a writer.
 *
 * @param writer The writer.
 * @param writeNs Whether to write the namespaces for this element to the xml writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer writeNamespaces: (BOOL) writeNs
{
  [self writeXMLType: writer];
}

@end /* implementation JAXBBasicXMLNode (JAXB) */

/**
 * Declaration of the JAXB type for a string.
 */
@interface NSString (JAXBType) <JAXBType>
@end

/**
 * Implementation of the JAXB type for a string.
 */
@implementation NSString (JAXBType)

/**
 * Read the XML type from the reader.
 *
 * @param reader The reader.
 * @return The NSString that was read from the reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  return [NSString stringWithUTF8String: (const char *) xmlTextReaderReadEntireNodeValue(reader)];
}

/**
 * Read an XML type from an XML reader into an existing instance.
 *
 * @param reader The reader.
 * @param existing The existing instance into which to read values.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  [NSException raise: @"XMLReadError"
               format: @"An existing string cannot be modified."];
  return nil;
}

/**
 * Write the NSString to the writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  xmlTextWriterWriteString(writer, BAD_CAST [self UTF8String]);
}

@end /*NSString (JAXBType)*/



/**
 * Declaration of the JAXB type for a big number.
 */
@interface NSNumber (JAXBType) <JAXBType>
@end

/**
 * Implementation of the JAXB type for a big number.
 */
@implementation NSNumber (JAXBType)

/**
 * Read the XML type from the reader.
 *
 * @param reader The reader.
 * @return The NSNumber that was read from the reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  return [NSNumber numberWithLongLong: [[NSString stringWithUTF8String: (const char *) xmlTextReaderReadEntireNodeValue(reader)] longLongValue]];
}

/**
 * Read an XML type from an XML reader into an existing instance.
 *
 * @param reader The reader.
 * @param existing The existing instance into which to read values.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  [NSException raise: @"XMLReadError"
               format: @"An existing number cannot be modified."];
  return nil;
}

/**
 * Write the NSNumber to the writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  xmlTextWriterWriteString(writer, BAD_CAST [[self description] UTF8String]);
}

@end /*NSNumber (JAXBType)*/



/**
 * Declaration of the JAXB type for a big number.
 */
@interface NSDecimalNumber (JAXBType) <JAXBType>
@end

/**
 * Implementation of the JAXB type for a big number.
 */
@implementation NSDecimalNumber (JAXBType)

/**
 * Read the XML type from the reader.
 *
 * @param reader The reader.
 * @return The NSDecimalNumber that was read from the reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  return [NSDecimalNumber decimalNumberWithString: [NSString stringWithUTF8String: (const char *) xmlTextReaderReadEntireNodeValue(reader)]];
}

/**
 * Read an XML type from an XML reader into an existing instance.
 *
 * @param reader The reader.
 * @param existing The existing instance into which to read values.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  [NSException raise: @"XMLReadError"
               format: @"An existing decimal number cannot be modified."];
  return nil;
}

/**
 * Write the NSDecimalNumber to the writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  xmlTextWriterWriteString(writer, BAD_CAST [[self description] UTF8String]);
}

@end /*NSDecimalNumber (JAXBType)*/



/**
 * Declaration of the JAXB type for a url.
 */
@interface NSURL (JAXBType) <JAXBType>
@end

/**
 * Implementation of the JAXB type for a url.
 */
@implementation NSURL (JAXBType)

/**
 * Read the XML type from the reader.
 *
 * @param reader The reader.
 * @return The NSURL that was read from the reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  return [NSURL URLWithString: [NSString stringWithUTF8String: (const char *) xmlTextReaderReadEntireNodeValue(reader)]];
}

/**
 * Read an XML type from an XML reader into an existing instance.
 *
 * @param reader The reader.
 * @param existing The existing instance into which to read values.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  [NSException raise: @"XMLReadError"
               format: @"An existing url cannot be modified."];
  return nil;
}

/**
 * Write the NSURL to the writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  xmlTextWriterWriteString(writer, BAD_CAST [[self absoluteString] UTF8String]);
}

@end /*NSURL (JAXBType)*/



/**
 * Declaration of the JAXB type for binary data.
 */
@interface NSData (JAXBType) <JAXBType>
@end

/**
 * Implementation of the JAXB type for a binary data.
 */
@implementation NSData (JAXBType)

/**
 * Read the XML type from the reader.
 *
 * @param reader The reader.
 * @return The NSData that was read from the reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  xmlChar *base64data = xmlTextReaderReadEntireNodeValue(reader);
  int len;
  unsigned char *data = _decode_base64(base64data, &len);
  NSData *wrappedData = [NSData dataWithBytesNoCopy: data length: len];
  free(base64data);
  return wrappedData;
}

/**
 * Read an XML type from an XML reader into an existing instance.
 *
 * @param reader The reader.
 * @param existing The existing instance into which to read values.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  [NSException raise: @"XMLReadError"
               format: @"An existing NSData cannot be modified."];
  return nil;
}

/**
 * Write the NSData to the writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  xmlChar *out = _encode_base64((unsigned char *)[self bytes], [self length]);
  xmlTextWriterWriteString(writer, out);
  free(out);
}

@end /*NSData (JAXBType)*/



/**
 * Declaration of the JAXB type for a big number.
 */
@interface NSDate (JAXBType) <JAXBType>
@end

/**
 * Implementation of the JAXB type for a big number.
 */
@implementation NSDate (JAXBType)

/**
 * Read the XML type from the reader.
 *
 * @param reader The reader.
 * @return The NSDate that was read from the reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  xmlChar *timevalue = xmlTextReaderReadEntireNodeValue(reader);
  NSInteger year = 0;
  NSUInteger month = 1, day = 1, hour = 0, minute = 0, second = 0;
  BOOL skip_time = NO;
  int index = 0, token_index = 0, len = xmlStrlen(timevalue), offset_hour = 0, offset_minute = 0;
  char token[len];

  if (len > (index + 5) && timevalue[index + 4] == '-') {
    //assume we're at yyyy-MM-dd
    token_index = 0;
    while (index < len && timevalue[index] != '-') {
      token[token_index++] = timevalue[index++];
    }
    token[token_index] = '\0';
    if (token_index != 4) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read dateTime %s; invalid year: %s", timevalue, token];
    }
    year = atoi(token);
    index++;

    //go to next '-' character.
    token_index = 0;
    while (index < len && timevalue[index] != '-') {
      token[token_index++] = timevalue[index++];
    }
    token[token_index] = '\0';
    if (token_index != 2) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read dateTime %s; invalid month: %s", timevalue, token];
    }
    month = atoi(token);
    index++;

    //go to 'T', 'Z', '+', or '-' character.
    token_index = 0;
    while (index < len && timevalue[index] != 'T' && timevalue[index] != 'Z' && timevalue[index] != '-' && timevalue[index] != '+') {
      token[token_index++] = timevalue[index++];
    }
    token[token_index] = '\0';
    if (token_index != 2) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read dateTime %s; invalid day of month: %s", timevalue, token];
    }
    day = atoi(token);
    if (timevalue[index] != 'T') {
      skip_time = YES;
    }
    if (timevalue[index] != '-') {
      index++;
    }
  }

  if (skip_time == NO || (len > (index + 3) && timevalue[index + 2] == ':')) {
    //assume we're at HH:mm:ss

    //go to ':' character.
    token_index = 0;
    while (index < len && timevalue[index] != ':') {
      token[token_index++] = timevalue[index++];
    }
    token[token_index] = '\0';
    if (token_index != 2) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read dateTime %s; invalid hour: %s", timevalue, token];
    }
    hour = atoi(token);
    index++;

    //go to ':' character.
    token_index = 0;
    while (index < len && timevalue[index] != ':') {
      token[token_index++] = timevalue[index++];
    }
    token[token_index] = '\0';
    if (token_index != 2) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read dateTime %s; invalid minute: %s", timevalue, token];
    }
    minute = atoi(token);
    index++;

    //go to '+' or '-' or 'Z' character.
    token_index = 0;
    while (index < len && timevalue[index] != '+' && timevalue[index] != '-' && timevalue[index] != 'Z') {
      token[token_index++] = timevalue[index++];
    }
    token[token_index] = '\0';
    if (token_index == 0) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read dateTime %s; invalid seconds: %s", timevalue, token];
    }
    second = (NSUInteger) atof(token);
    if (timevalue[index] != '-') {
      index++;
    }
  }

  //go to ':' character.
  token_index = 0;
  while (index < len && timevalue[index] != ':') {
    token[token_index++] = timevalue[index++];
  }
  token[token_index] = '\0';
  offset_hour += atoi(token);
  index++;

  //go to end.
  token_index = 0;
  while (index < len) {
    token[token_index++] = timevalue[index++];
  }
  token[token_index] = '\0';
  offset_minute += atoi(token);

  //YYYY-MM-DD HH:MM:SS -HHHMM
  free(timevalue);
  return [NSDate dateWithString: [NSString stringWithFormat: @"%04i-%02i-%02i %02i:%02i:%02i %+03i%02i", year, month, day, hour, minute, second, offset_hour, offset_minute]];
}

/**
 * Read an XML type from an XML reader into an existing instance.
 *
 * @param reader The reader.
 * @param existing The existing instance into which to read values.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  [NSException raise: @"XMLReadError"
               format: @"An existing date cannot be modified."];
  return nil;
}

/**
 * Write the NSDate to the writer.
 *
 * @param writer The writer.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  NSDateFormatter *formatter = [[NSDateFormatter alloc] initWithDateFormat: @"%Y-%m-%dT%H:%M:%S %z" allowNaturalLanguage: NO];
  xmlChar *timevalue = BAD_CAST [[formatter stringForObjectValue: self] UTF8String];
  timevalue[19] = timevalue[20];
  timevalue[20] = timevalue[21];
  timevalue[21] = timevalue[22];
  timevalue[22] = ':';
  xmlTextWriterWriteString(writer, timevalue);
  [formatter release];
}
@end /*NSDate (JAXBType)*/

#endif /* ENUNCIATE_OBJC_CLASSES */

#ifndef ENUNCIATE_XML_OBJC_PRIMITIVE_FUNCTIONS
#define ENUNCIATE_XML_OBJC_PRIMITIVE_FUNCTIONS

/*******************boolean************************************/

/**
 * Read a boolean value from the reader.
 *
 * @param reader The reader (pointing at a node with a value).
 * @return YES if "true" was read. NO otherwise.
 */
static BOOL *xmlTextReaderReadBooleanType(xmlTextReaderPtr reader) {
  xmlChar *nodeValue = xmlTextReaderReadEntireNodeValue(reader);
  BOOL *value = malloc(sizeof(BOOL));
  *value = (xmlStrcmp(BAD_CAST "true", nodeValue) == 0) ? YES : NO;
  free(nodeValue);
  return value;
}

/**
 * Write a boolean value to the writer.
 *
 * @param writer The writer.
 * @param value The value to be written.
 * @return the bytes written (may be 0 because of buffering) or -1 in case of error.
 */
static int xmlTextWriterWriteBooleanType(xmlTextWriterPtr writer, BOOL *value) {
  if (*value) {
    return xmlTextWriterWriteString(writer, BAD_CAST "false");
  }
  else {
    return xmlTextWriterWriteString(writer, BAD_CAST "true");
  }
}

/*******************byte************************************/

/**
 * Read a byte value from the reader.
 *
 * @param reader The reader (pointing at a node with a value).
 * @return the byte.
 */
static unsigned char *xmlTextReaderReadByteType(xmlTextReaderPtr reader) {
  xmlChar *nodeValue = xmlTextReaderReadEntireNodeValue(reader);
  unsigned char *value = malloc(sizeof(unsigned char));
  *value = (unsigned char) atoi((char *) nodeValue);
  free(nodeValue);
  return value;
}

/**
 * Write a byte value to the writer.
 *
 * @param writer The writer.
 * @param value The value to be written.
 * @return the bytes written (may be 0 because of buffering) or -1 in case of error.
 */
static int xmlTextWriterWriteByteType(xmlTextWriterPtr writer, unsigned char *value) {
  return xmlTextWriterWriteFormatString(writer, "%i", *value);
}

/*******************double************************************/

/**
 * Read a double value from the reader.
 *
 * @param reader The reader (pointing at a node with a value).
 * @return the double.
 */
static double *xmlTextReaderReadDoubleType(xmlTextReaderPtr reader) {
  xmlChar *nodeValue = xmlTextReaderReadEntireNodeValue(reader);
  double *value = malloc(sizeof(double));
  *value = atof((char *) nodeValue);
  free(nodeValue);
  return value;
}

/**
 * Write a double value to the writer.
 *
 * @param writer The writer.
 * @param value The value to be written.
 * @return the bytes written (may be 0 because of buffering) or -1 in case of error.
 */
static int xmlTextWriterWriteDoubleType(xmlTextWriterPtr writer, double *value) {
  return xmlTextWriterWriteFormatString(writer, "%f", *value);
}

/*******************float************************************/

/**
 * Read a float value from the reader.
 *
 * @param reader The reader (pointing at a node with a value).
 * @return the float.
 */
static float *xmlTextReaderReadFloatType(xmlTextReaderPtr reader) {
  xmlChar *nodeValue = xmlTextReaderReadEntireNodeValue(reader);
  float *value = malloc(sizeof(float));
  *value = atof((char *)nodeValue);
  free(nodeValue);
  return value;
}

/**
 * Write a float value to the writer.
 *
 * @param writer The writer.
 * @param value The value to be written.
 * @return the bytes written (may be 0 because of buffering) or -1 in case of error.
 */
static int xmlTextWriterWriteFloatType(xmlTextWriterPtr writer, float *value) {
  return xmlTextWriterWriteFormatString(writer, "%f", *value);
}

/*******************int************************************/

/**
 * Read a int value from the reader.
 *
 * @param reader The reader (pointing at a node with a value).
 * @param value The value to be written.
 * @return the int.
 */
static int *xmlTextReaderReadIntType(xmlTextReaderPtr reader) {
  xmlChar *nodeValue = xmlTextReaderReadEntireNodeValue(reader);
  int *value = malloc(sizeof(int));
  *value = atoi((char *)nodeValue);
  free(nodeValue);
  return value;
}

/**
 * Write a int value to the writer.
 *
 * @param writer The writer.
 * @param value The value to be written.
 * @return the bytes written (may be 0 because of buffering) or -1 in case of error.
 */
static int xmlTextWriterWriteIntType(xmlTextWriterPtr writer, int *value) {
  return xmlTextWriterWriteFormatString(writer, "%i", *value);
}

/*******************long************************************/

/**
 * Read a long value from the reader.
 *
 * @param reader The reader (pointing at a node with a value).
 * @return the long.
 */
static long *xmlTextReaderReadLongType(xmlTextReaderPtr reader) {
  xmlChar *nodeValue = xmlTextReaderReadEntireNodeValue(reader);
  long *value = malloc(sizeof(long));
  *value = atol((char *)nodeValue);
  free(nodeValue);
  return value;
}

/**
 * Write a long value to the writer.
 *
 * @param writer The writer.
 * @param value The value to be written.
 * @return the bytes written (may be 0 because of buffering) or -1 in case of error.
 */
static int xmlTextWriterWriteLongType(xmlTextWriterPtr writer, long *value) {
  return xmlTextWriterWriteFormatString(writer, "%ld", *value);
}

/*******************short************************************/

/**
 * Read a short value from the reader.
 *
 * @param reader The reader (pointing at a node with a value).
 * @return the short.
 */
static short *xmlTextReaderReadShortType(xmlTextReaderPtr reader) {
  xmlChar *nodeValue = xmlTextReaderReadEntireNodeValue(reader);
  short *value = malloc(sizeof(short));
  *value = atoi((char *)nodeValue);
  free(nodeValue);
  return value;
}

/**
 * Write a short value to the writer.
 *
 * @param writer The writer.
 * @param value The value to be written.
 * @return the bytes written (may be 0 because of buffering) or -1 in case of error.
 */
static int xmlTextWriterWriteShortType(xmlTextWriterPtr writer, short *value) {
  return xmlTextWriterWriteFormatString(writer, "%hi", *value);
}

/*******************char************************************/

/**
 * Read a character value from the reader.
 *
 * @param reader The reader (pointing at a node with a value).
 * @return the character.
 */
static xmlChar *xmlTextReaderReadCharacterType(xmlTextReaderPtr reader) {
  return xmlTextReaderReadEntireNodeValue(reader);
}

/**
 * Write a character value to the writer.
 *
 * @param writer The writer.
 * @param value The value to be written.
 * @return the bytes written (may be 0 because of buffering) or -1 in case of error.
 */
static int xmlTextWriterWriteCharacterType(xmlTextWriterPtr writer, xmlChar *value) {
  return xmlTextWriterWriteString(writer, value);
}

#endif /* ENUNCIATE_XML_OBJC_PRIMITIVE_FUNCTIONS */

#ifndef DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO_M
#define DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO_M

/**
 * (no documentation provided)
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO

/**
 * the emailAddressAttribute
 */
- (NSString *) emailAddressAttribute
{
  return _emailAddressAttribute;
}

/**
 * the emailAddressAttribute
 */
- (void) setEmailAddressAttribute: (NSString *) newEmailAddressAttribute
{
  [newEmailAddressAttribute retain];
  [_emailAddressAttribute release];
  _emailAddressAttribute = newEmailAddressAttribute;
}

/**
 * the groupBaseDn
 */
- (NSString *) groupBaseDn
{
  return _groupBaseDn;
}

/**
 * the groupBaseDn
 */
- (void) setGroupBaseDn: (NSString *) newGroupBaseDn
{
  [newGroupBaseDn retain];
  [_groupBaseDn release];
  _groupBaseDn = newGroupBaseDn;
}

/**
 * the groupIdAttribute
 */
- (NSString *) groupIdAttribute
{
  return _groupIdAttribute;
}

/**
 * the groupIdAttribute
 */
- (void) setGroupIdAttribute: (NSString *) newGroupIdAttribute
{
  [newGroupIdAttribute retain];
  [_groupIdAttribute release];
  _groupIdAttribute = newGroupIdAttribute;
}

/**
 * the groupMemberAttribute
 */
- (NSString *) groupMemberAttribute
{
  return _groupMemberAttribute;
}

/**
 * the groupMemberAttribute
 */
- (void) setGroupMemberAttribute: (NSString *) newGroupMemberAttribute
{
  [newGroupMemberAttribute retain];
  [_groupMemberAttribute release];
  _groupMemberAttribute = newGroupMemberAttribute;
}

/**
 * the groupMemberFormat
 */
- (NSString *) groupMemberFormat
{
  return _groupMemberFormat;
}

/**
 * the groupMemberFormat
 */
- (void) setGroupMemberFormat: (NSString *) newGroupMemberFormat
{
  [newGroupMemberFormat retain];
  [_groupMemberFormat release];
  _groupMemberFormat = newGroupMemberFormat;
}

/**
 * the groupObjectClass
 */
- (NSString *) groupObjectClass
{
  return _groupObjectClass;
}

/**
 * the groupObjectClass
 */
- (void) setGroupObjectClass: (NSString *) newGroupObjectClass
{
  [newGroupObjectClass retain];
  [_groupObjectClass release];
  _groupObjectClass = newGroupObjectClass;
}

/**
 * the userPasswordAttribute
 */
- (NSString *) userPasswordAttribute
{
  return _userPasswordAttribute;
}

/**
 * the userPasswordAttribute
 */
- (void) setUserPasswordAttribute: (NSString *) newUserPasswordAttribute
{
  [newUserPasswordAttribute retain];
  [_userPasswordAttribute release];
  _userPasswordAttribute = newUserPasswordAttribute;
}

/**
 * the userIdAttribute
 */
- (NSString *) userIdAttribute
{
  return _userIdAttribute;
}

/**
 * the userIdAttribute
 */
- (void) setUserIdAttribute: (NSString *) newUserIdAttribute
{
  [newUserIdAttribute retain];
  [_userIdAttribute release];
  _userIdAttribute = newUserIdAttribute;
}

/**
 * the userObjectClass
 */
- (NSString *) userObjectClass
{
  return _userObjectClass;
}

/**
 * the userObjectClass
 */
- (void) setUserObjectClass: (NSString *) newUserObjectClass
{
  [newUserObjectClass retain];
  [_userObjectClass release];
  _userObjectClass = newUserObjectClass;
}

/**
 * the userBaseDn
 */
- (NSString *) userBaseDn
{
  return _userBaseDn;
}

/**
 * the userBaseDn
 */
- (void) setUserBaseDn: (NSString *) newUserBaseDn
{
  [newUserBaseDn retain];
  [_userBaseDn release];
  _userBaseDn = newUserBaseDn;
}

/**
 * the userRealNameAttribute
 */
- (NSString *) userRealNameAttribute
{
  return _userRealNameAttribute;
}

/**
 * the userRealNameAttribute
 */
- (void) setUserRealNameAttribute: (NSString *) newUserRealNameAttribute
{
  [newUserRealNameAttribute retain];
  [_userRealNameAttribute release];
  _userRealNameAttribute = newUserRealNameAttribute;
}

/**
 * the ldapGroupsAsRoles
 */
- (BOOL) ldapGroupsAsRoles
{
  return _ldapGroupsAsRoles;
}

/**
 * the ldapGroupsAsRoles
 */
- (void) setLdapGroupsAsRoles: (BOOL) newLdapGroupsAsRoles
{
  _ldapGroupsAsRoles = newLdapGroupsAsRoles;
}

/**
 * the userSubtree
 */
- (BOOL) userSubtree
{
  return _userSubtree;
}

/**
 * the userSubtree
 */
- (void) setUserSubtree: (BOOL) newUserSubtree
{
  _userSubtree = newUserSubtree;
}

/**
 * the groupSubtree
 */
- (BOOL) groupSubtree
{
  return _groupSubtree;
}

/**
 * the groupSubtree
 */
- (void) setGroupSubtree: (BOOL) newGroupSubtree
{
  _groupSubtree = newGroupSubtree;
}

/**
 * (no documentation provided)
 */
- (NSString *) userMemberOfAttribute
{
  return _userMemberOfAttribute;
}

/**
 * (no documentation provided)
 */
- (void) setUserMemberOfAttribute: (NSString *) newUserMemberOfAttribute
{
  [newUserMemberOfAttribute retain];
  [_userMemberOfAttribute release];
  _userMemberOfAttribute = newUserMemberOfAttribute;
}

/**
 * the filter to select specific users
 */
- (NSString *) ldapFilter
{
  return _ldapFilter;
}

/**
 * the filter to select specific users
 */
- (void) setLdapFilter: (NSString *) newLdapFilter
{
  [newLdapFilter retain];
  [_ldapFilter release];
  _ldapFilter = newLdapFilter;
}

- (void) dealloc
{
  [self setEmailAddressAttribute: nil];
  [self setGroupBaseDn: nil];
  [self setGroupIdAttribute: nil];
  [self setGroupMemberAttribute: nil];
  [self setGroupMemberFormat: nil];
  [self setGroupObjectClass: nil];
  [self setUserPasswordAttribute: nil];
  [self setUserIdAttribute: nil];
  [self setUserObjectClass: nil];
  [self setUserBaseDn: nil];
  [self setUserRealNameAttribute: nil];
  [self setUserMemberOfAttribute: nil];
  [self setLdapFilter: nil];
  [super dealloc];
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO */

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO (JAXB) <JAXBReading, JAXBWriting, JAXBType>

@end /*interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO (JAXB)*/

/**
 * Internal, private implementation for JAXB reading and writing.
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO (JAXB)

/**
 * Read an instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO defined by the XML reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO *_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO = [[NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO alloc] init];
  NS_DURING
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO initWithReader: reader];
  }
  NS_HANDLER
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO dealloc];
    _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO autorelease];
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO;
}

/**
 * Initialize this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO according to
 * the XML being read from the reader.
 *
 * @param reader The reader.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  return [super initWithReader: reader];
}

/**
 * Write the XML for this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO to the writer.
 * Note that since we're only writing the XML type,
 * No start/end element will be written.
 *
 * @param reader The reader.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [super writeXMLType:writer];
}

//documentation inherited.
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  void *_child_accessor;

  if ([super readJAXBAttribute: reader]) {
    return YES;
  }

  return NO;
}

//documentation inherited.
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return [super readJAXBValue: reader];
}

//documentation inherited.
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  id __child;
  void *_child_accessor;
  int status, depth;

  if ([super readJAXBChildElement: reader]) {
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "emailAddressAttribute", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}emailAddressAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}emailAddressAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setEmailAddressAttribute: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "groupBaseDn", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}groupBaseDn of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}groupBaseDn of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setGroupBaseDn: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "groupIdAttribute", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}groupIdAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}groupIdAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setGroupIdAttribute: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "groupMemberAttribute", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}groupMemberAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}groupMemberAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setGroupMemberAttribute: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "groupMemberFormat", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}groupMemberFormat of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}groupMemberFormat of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setGroupMemberFormat: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "groupObjectClass", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}groupObjectClass of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}groupObjectClass of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setGroupObjectClass: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "userPasswordAttribute", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}userPasswordAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}userPasswordAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setUserPasswordAttribute: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "userIdAttribute", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}userIdAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}userIdAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setUserIdAttribute: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "userObjectClass", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}userObjectClass of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}userObjectClass of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setUserObjectClass: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "userBaseDn", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}userBaseDn of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}userBaseDn of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setUserBaseDn: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "userRealNameAttribute", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}userRealNameAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}userRealNameAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setUserRealNameAttribute: __child];
    return YES;
  } //end "if choice"


  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "ldapGroupsAsRoles", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

    _child_accessor = xmlTextReaderReadBooleanType(reader);
    if (_child_accessor == NULL) {
      //panic: unable to return the value for some reason.
      [self dealloc];
      [NSException raise: @"XMLReadError"
                   format: @"Error reading element value."];
    }
    [self setLdapGroupsAsRoles: *((BOOL*) _child_accessor)];
    free(_child_accessor);
    return YES;
  }

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "userSubtree", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

    _child_accessor = xmlTextReaderReadBooleanType(reader);
    if (_child_accessor == NULL) {
      //panic: unable to return the value for some reason.
      [self dealloc];
      [NSException raise: @"XMLReadError"
                   format: @"Error reading element value."];
    }
    [self setUserSubtree: *((BOOL*) _child_accessor)];
    free(_child_accessor);
    return YES;
  }

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "groupSubtree", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

    _child_accessor = xmlTextReaderReadBooleanType(reader);
    if (_child_accessor == NULL) {
      //panic: unable to return the value for some reason.
      [self dealloc];
      [NSException raise: @"XMLReadError"
                   format: @"Error reading element value."];
    }
    [self setGroupSubtree: *((BOOL*) _child_accessor)];
    free(_child_accessor);
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "userMemberOfAttribute", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}userMemberOfAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}userMemberOfAttribute of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setUserMemberOfAttribute: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "ldapFilter", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}ldapFilter of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}ldapFilter of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setLdapFilter: __child];
    return YES;
  } //end "if choice"


  return NO;
}

//documentation inherited.
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return [super readUnknownJAXBChildElement: reader];
}

//documentation inherited.
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
  [super readUnknownJAXBAttribute: reader];
}

//documentation inherited.
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  int status;

  [super writeJAXBAttributes: writer];

}

//documentation inherited.
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  [super writeJAXBValue: writer];
}

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  int status;
  id __item;
  NSEnumerator *__enumerator;

  [super writeJAXBChildElements: writer];

  if ([self emailAddressAttribute]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "emailAddressAttribute", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}emailAddressAttribute."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}emailAddressAttribute...");
#endif
    [[self emailAddressAttribute] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}emailAddressAttribute...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}emailAddressAttribute."];
    }
  }
  if ([self groupBaseDn]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "groupBaseDn", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}groupBaseDn."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}groupBaseDn...");
#endif
    [[self groupBaseDn] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}groupBaseDn...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}groupBaseDn."];
    }
  }
  if ([self groupIdAttribute]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "groupIdAttribute", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}groupIdAttribute."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}groupIdAttribute...");
#endif
    [[self groupIdAttribute] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}groupIdAttribute...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}groupIdAttribute."];
    }
  }
  if ([self groupMemberAttribute]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "groupMemberAttribute", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}groupMemberAttribute."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}groupMemberAttribute...");
#endif
    [[self groupMemberAttribute] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}groupMemberAttribute...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}groupMemberAttribute."];
    }
  }
  if ([self groupMemberFormat]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "groupMemberFormat", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}groupMemberFormat."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}groupMemberFormat...");
#endif
    [[self groupMemberFormat] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}groupMemberFormat...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}groupMemberFormat."];
    }
  }
  if ([self groupObjectClass]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "groupObjectClass", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}groupObjectClass."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}groupObjectClass...");
#endif
    [[self groupObjectClass] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}groupObjectClass...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}groupObjectClass."];
    }
  }
  if ([self userPasswordAttribute]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userPasswordAttribute", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}userPasswordAttribute."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}userPasswordAttribute...");
#endif
    [[self userPasswordAttribute] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}userPasswordAttribute...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}userPasswordAttribute."];
    }
  }
  if ([self userIdAttribute]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userIdAttribute", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}userIdAttribute."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}userIdAttribute...");
#endif
    [[self userIdAttribute] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}userIdAttribute...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}userIdAttribute."];
    }
  }
  if ([self userObjectClass]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userObjectClass", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}userObjectClass."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}userObjectClass...");
#endif
    [[self userObjectClass] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}userObjectClass...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}userObjectClass."];
    }
  }
  if ([self userBaseDn]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userBaseDn", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}userBaseDn."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}userBaseDn...");
#endif
    [[self userBaseDn] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}userBaseDn...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}userBaseDn."];
    }
  }
  if ([self userRealNameAttribute]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userRealNameAttribute", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}userRealNameAttribute."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}userRealNameAttribute...");
#endif
    [[self userRealNameAttribute] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}userRealNameAttribute...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}userRealNameAttribute."];
    }
  }
  if (YES) { //always write the primitive element...
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "ldapGroupsAsRoles", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}ldapGroupsAsRoles."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}ldapGroupsAsRoles...");
#endif
    status = xmlTextWriterWriteBooleanType(writer, &_ldapGroupsAsRoles);
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}ldapGroupsAsRoles...");
#endif
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing child element {}ldapGroupsAsRoles."];
    }

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}ldapGroupsAsRoles."];
    }
  }
  if (YES) { //always write the primitive element...
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userSubtree", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}userSubtree."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}userSubtree...");
#endif
    status = xmlTextWriterWriteBooleanType(writer, &_userSubtree);
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}userSubtree...");
#endif
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing child element {}userSubtree."];
    }

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}userSubtree."];
    }
  }
  if (YES) { //always write the primitive element...
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "groupSubtree", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}groupSubtree."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}groupSubtree...");
#endif
    status = xmlTextWriterWriteBooleanType(writer, &_groupSubtree);
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}groupSubtree...");
#endif
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing child element {}groupSubtree."];
    }

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}groupSubtree."];
    }
  }
  if ([self userMemberOfAttribute]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userMemberOfAttribute", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}userMemberOfAttribute."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}userMemberOfAttribute...");
#endif
    [[self userMemberOfAttribute] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}userMemberOfAttribute...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}userMemberOfAttribute."];
    }
  }
  if ([self ldapFilter]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "ldapFilter", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}ldapFilter."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}ldapFilter...");
#endif
    [[self ldapFilter] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}ldapFilter...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}ldapFilter."];
    }
  }
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO (JAXB) */

#endif /* DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO_M */
#ifndef DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse_M
#define DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse_M

/**
 * (no documentation provided)
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse

/**
 * the ldapUserRoleMappings
 */
- (NSArray *) ldapUserRoleMappings
{
  return _ldapUserRoleMappings;
}

/**
 * the ldapUserRoleMappings
 */
- (void) setLdapUserRoleMappings: (NSArray *) newLdapUserRoleMappings
{
  [newLdapUserRoleMappings retain];
  [_ldapUserRoleMappings release];
  _ldapUserRoleMappings = newLdapUserRoleMappings;
}

- (void) dealloc
{
  [self setLdapUserRoleMappings: nil];
  [super dealloc];
}

//documentation inherited.
+ (id<EnunciateXML>) readFromXML: (NSData *) xml
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse *_nEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse;
  xmlTextReaderPtr reader = xmlReaderForMemory([xml bytes], [xml length], NULL, NULL, 0);
  if (reader == NULL) {
    [NSException raise: @"XMLReadError"
                 format: @"Error instantiating an XML reader."];
    return nil;
  }

  _nEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse = (NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse *) [NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse readXMLElement: reader];
  xmlFreeTextReader(reader); //free the reader
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse;
}

//documentation inherited.
- (NSData *) writeToXML
{
  xmlBufferPtr buf;
  xmlTextWriterPtr writer;
  int rc;
  NSData *data;

  buf = xmlBufferCreate();
  if (buf == NULL) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML buffer."];
    return nil;
  }

  writer = xmlNewTextWriterMemory(buf, 0);
  if (writer == NULL) {
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML writer."];
    return nil;
  }

  rc = xmlTextWriterStartDocument(writer, NULL, "utf-8", NULL);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML start document."];
    return nil;
  }

  NS_DURING
  {
    [self writeXMLElement: writer];
  }
  NS_HANDLER
  {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [localException raise];
  }
  NS_ENDHANDLER

  rc = xmlTextWriterEndDocument(writer);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML end document."];
    return nil;
  }

  xmlFreeTextWriter(writer);
  data = [NSData dataWithBytes: buf->content length: buf->use];
  xmlBufferFree(buf);
  return data;
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse */

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse (JAXB) <JAXBReading, JAXBWriting, JAXBType, JAXBElement>

@end /*interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse (JAXB)*/

/**
 * Internal, private implementation for JAXB reading and writing.
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse (JAXB)

/**
 * Read an instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse defined by the XML reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse *_nEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse = [[NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse alloc] init];
  NS_DURING
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse initWithReader: reader];
  }
  NS_HANDLER
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse dealloc];
    _nEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse autorelease];
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse;
}

/**
 * Initialize this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse according to
 * the XML being read from the reader.
 *
 * @param reader The reader.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  return [super initWithReader: reader];
}

/**
 * Write the XML for this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse to the writer.
 * Note that since we're only writing the XML type,
 * No start/end element will be written.
 *
 * @param reader The reader.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [super writeXMLType:writer];
}

/**
 * Reads a NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse from an XML reader. The element to be read is
 * "userList".
 *
 * @param reader The XML reader.
 * @return The NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse.
 */
+ (id<JAXBElement>) readXMLElement: (xmlTextReaderPtr) reader {
  int status;
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse *_ldapUserListResponse = nil;

  if (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT) {
    status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
    if (status < 1) {
      [NSException raise: @"XMLReadError"
                   format: @"Error advancing the reader to start element userList."];
    }
  }

  if (xmlStrcmp(BAD_CAST "userList", xmlTextReaderConstLocalName(reader)) == 0
      && xmlTextReaderConstNamespaceUri(reader) == NULL) {
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read root element {}userList.");
#endif
    _ldapUserListResponse = (NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse *)[NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Successfully read root element {}userList.");
#endif
  }
  else {
    if (xmlTextReaderConstNamespaceUri(reader) == NULL) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse. Expected element userList. Current element: {}%s", xmlTextReaderConstLocalName(reader)];
    }
    else {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse. Expected element userList. Current element: {%s}%s\n", xmlTextReaderConstNamespaceUri(reader), xmlTextReaderConstLocalName(reader)];
    }
  }

  return _ldapUserListResponse;
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse to XML under element name "userList".
 * The namespace declarations for the element will be written.
 *
 * @param writer The XML writer.
 * @param _ldapUserListResponse The LdapUserListResponse to write.
 * @return 1 if successful, 0 otherwise.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer
{
  [self writeXMLElement: writer writeNamespaces: YES];
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse to an XML writer.
 *
 * @param writer The writer.
 * @param writeNs Whether to write the namespaces for this element to the xml writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer writeNamespaces: (BOOL) writeNs
{
  int rc = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userList", NULL);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing start element {}userList. XML writer status: %i\n", rc];
  }

#if DEBUG_ENUNCIATE > 1
  NSLog(@"writing type {}ldapUserListResponse for root element {}userList...");
#endif
  [self writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
  NSLog(@"successfully wrote type {}ldapUserListResponse for root element {}userList...");
#endif
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing end element {}userList. XML writer status: %i\n", rc];
  }
}

//documentation inherited.
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  void *_child_accessor;

  if ([super readJAXBAttribute: reader]) {
    return YES;
  }

  return NO;
}

//documentation inherited.
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return [super readJAXBValue: reader];
}

//documentation inherited.
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  id __child;
  void *_child_accessor;
  int status, depth;

  if ([super readJAXBChildElement: reader]) {
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "data", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

    if (xmlTextReaderIsEmptyElement(reader) == 0) {
#if DEBUG_ENUNCIATE > 1
      NSLog(@"Unwrapping wrapper element {}data...");
#endif
      //start wrapper element "{}data"
      depth = xmlTextReaderDepth(reader);
      status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
      while (xmlTextReaderDepth(reader) > depth) {
        if (status < 1) {
          //panic: XML read error.
          [self dealloc];
          [NSException raise: @"XMLReadError"
                       format: @"Failure to advance to next wrapped child element."];
        }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "user", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}user of type {}user.");
#endif

    __child = [NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}user of type {}user.");
#endif

    if ([self ldapUserRoleMappings]) {
      [self setLdapUserRoleMappings: [[self ldapUserRoleMappings] arrayByAddingObject: __child]];
    }
    else {
      [self setLdapUserRoleMappings: [NSArray arrayWithObject: __child]];
    }
    continue; //continue "while in wrapper element" loop
  } //end "if choice"

        status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
#if DEBUG_ENUNCIATE > 1
        NSLog(@"successfully unwrapped wrapper element {}data...");
#endif
      } // end "while in wrapper element" loop

      if (status < 1) {
        //panic: XML read error.
        [self dealloc];
        [NSException raise: @"XMLReadError"
                     format: @"Failure to advance to end wrapper element."];
      }
      else {
        //end wrapper element "{}data"
        status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
      }
    } //end "if empty element" clause
    else {
      __child = [NSArray array];
      [self setLdapUserRoleMappings: (NSArray*) __child];
    }

    return YES;
  } // end "if wrapper element" clause

  return NO;
}

//documentation inherited.
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return [super readUnknownJAXBChildElement: reader];
}

//documentation inherited.
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
  [super readUnknownJAXBAttribute: reader];
}

//documentation inherited.
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  int status;

  [super writeJAXBAttributes: writer];

}

//documentation inherited.
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  [super writeJAXBValue: writer];
}

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  int status;
  id __item;
  NSEnumerator *__enumerator;

  [super writeJAXBChildElements: writer];

  if ([self ldapUserRoleMappings]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "data", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start wrapper element {}data."];
    }
    __enumerator = [[self ldapUserRoleMappings] objectEnumerator];

    while ( (__item = [__enumerator nextObject]) ) {
      status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "user", NULL);
      if (status < 0) {
        [NSException raise: @"XMLWriteError"
                     format: @"Error writing start child element {}user."];
      }

#if DEBUG_ENUNCIATE > 1
      NSLog(@"writing element {}user...");
#endif
      [__item writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
      NSLog(@"successfully wrote element {}user...");
#endif

      status = xmlTextWriterEndElement(writer);
      if (status < 0) {
        [NSException raise: @"XMLWriteError"
                     format: @"Error writing end child element {}user."];
      }
    } //end item iterator.

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end wrapper element {}data."];
    }
  }
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse (JAXB) */

#endif /* DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserListResponse_M */
#ifndef DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest_M
#define DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest_M

/**
 * (no documentation provided)
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest

/**
 * the ldapConnectionInfoDTO
 */
- (NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO *) data
{
  return _data;
}

/**
 * the ldapConnectionInfoDTO
 */
- (void) setData: (NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO *) newData
{
  [newData retain];
  [_data release];
  _data = newData;
}

- (void) dealloc
{
  [self setData: nil];
  [super dealloc];
}

//documentation inherited.
+ (id<EnunciateXML>) readFromXML: (NSData *) xml
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest *_nEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest;
  xmlTextReaderPtr reader = xmlReaderForMemory([xml bytes], [xml length], NULL, NULL, 0);
  if (reader == NULL) {
    [NSException raise: @"XMLReadError"
                 format: @"Error instantiating an XML reader."];
    return nil;
  }

  _nEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest = (NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest *) [NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest readXMLElement: reader];
  xmlFreeTextReader(reader); //free the reader
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest;
}

//documentation inherited.
- (NSData *) writeToXML
{
  xmlBufferPtr buf;
  xmlTextWriterPtr writer;
  int rc;
  NSData *data;

  buf = xmlBufferCreate();
  if (buf == NULL) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML buffer."];
    return nil;
  }

  writer = xmlNewTextWriterMemory(buf, 0);
  if (writer == NULL) {
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML writer."];
    return nil;
  }

  rc = xmlTextWriterStartDocument(writer, NULL, "utf-8", NULL);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML start document."];
    return nil;
  }

  NS_DURING
  {
    [self writeXMLElement: writer];
  }
  NS_HANDLER
  {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [localException raise];
  }
  NS_ENDHANDLER

  rc = xmlTextWriterEndDocument(writer);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML end document."];
    return nil;
  }

  xmlFreeTextWriter(writer);
  data = [NSData dataWithBytes: buf->content length: buf->use];
  xmlBufferFree(buf);
  return data;
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest */

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest (JAXB) <JAXBReading, JAXBWriting, JAXBType, JAXBElement>

@end /*interface NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest (JAXB)*/

/**
 * Internal, private implementation for JAXB reading and writing.
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest (JAXB)

/**
 * Read an instance of NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest defined by the XML reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest *_nEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest = [[NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest alloc] init];
  NS_DURING
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest initWithReader: reader];
  }
  NS_HANDLER
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest dealloc];
    _nEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [_nEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest autorelease];
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest;
}

/**
 * Initialize this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest according to
 * the XML being read from the reader.
 *
 * @param reader The reader.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  return [super initWithReader: reader];
}

/**
 * Write the XML for this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest to the writer.
 * Note that since we're only writing the XML type,
 * No start/end element will be written.
 *
 * @param reader The reader.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [super writeXMLType:writer];
}

/**
 * Reads a NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest from an XML reader. The element to be read is
 * "connectionInfoTest".
 *
 * @param reader The XML reader.
 * @return The NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest.
 */
+ (id<JAXBElement>) readXMLElement: (xmlTextReaderPtr) reader {
  int status;
  NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest *_ldapAuthenticationTestRequest = nil;

  if (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT) {
    status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
    if (status < 1) {
      [NSException raise: @"XMLReadError"
                   format: @"Error advancing the reader to start element connectionInfoTest."];
    }
  }

  if (xmlStrcmp(BAD_CAST "connectionInfoTest", xmlTextReaderConstLocalName(reader)) == 0
      && xmlTextReaderConstNamespaceUri(reader) == NULL) {
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read root element {}connectionInfoTest.");
#endif
    _ldapAuthenticationTestRequest = (NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest *)[NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Successfully read root element {}connectionInfoTest.");
#endif
  }
  else {
    if (xmlTextReaderConstNamespaceUri(reader) == NULL) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest. Expected element connectionInfoTest. Current element: {}%s", xmlTextReaderConstLocalName(reader)];
    }
    else {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest. Expected element connectionInfoTest. Current element: {%s}%s\n", xmlTextReaderConstNamespaceUri(reader), xmlTextReaderConstLocalName(reader)];
    }
  }

  return _ldapAuthenticationTestRequest;
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest to XML under element name "connectionInfoTest".
 * The namespace declarations for the element will be written.
 *
 * @param writer The XML writer.
 * @param _ldapAuthenticationTestRequest The LdapAuthenticationTestRequest to write.
 * @return 1 if successful, 0 otherwise.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer
{
  [self writeXMLElement: writer writeNamespaces: YES];
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest to an XML writer.
 *
 * @param writer The writer.
 * @param writeNs Whether to write the namespaces for this element to the xml writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer writeNamespaces: (BOOL) writeNs
{
  int rc = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "connectionInfoTest", NULL);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing start element {}connectionInfoTest. XML writer status: %i\n", rc];
  }

#if DEBUG_ENUNCIATE > 1
  NSLog(@"writing type {}ldapAuthenticationTestRequest for root element {}connectionInfoTest...");
#endif
  [self writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
  NSLog(@"successfully wrote type {}ldapAuthenticationTestRequest for root element {}connectionInfoTest...");
#endif
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing end element {}connectionInfoTest. XML writer status: %i\n", rc];
  }
}

//documentation inherited.
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  void *_child_accessor;

  if ([super readJAXBAttribute: reader]) {
    return YES;
  }

  return NO;
}

//documentation inherited.
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return [super readJAXBValue: reader];
}

//documentation inherited.
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  id __child;
  void *_child_accessor;
  int status, depth;

  if ([super readJAXBChildElement: reader]) {
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "data", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}data of type {}ldapConnectionInfoDTO.");
#endif
    __child = [NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}data of type {}ldapConnectionInfoDTO.");
#endif

    [self setData: __child];
    return YES;
  } //end "if choice"


  return NO;
}

//documentation inherited.
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return [super readUnknownJAXBChildElement: reader];
}

//documentation inherited.
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
  [super readUnknownJAXBAttribute: reader];
}

//documentation inherited.
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  int status;

  [super writeJAXBAttributes: writer];

}

//documentation inherited.
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  [super writeJAXBValue: writer];
}

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  int status;
  id __item;
  NSEnumerator *__enumerator;

  [super writeJAXBChildElements: writer];

  if ([self data]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "data", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}data."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}data...");
#endif
    [[self data] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}data...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}data."];
    }
  }
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest (JAXB) */

#endif /* DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapAuthenticationTestRequest_M */
#ifndef DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest_M
#define DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest_M

/**
 * (no documentation provided)
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest

/**
 * the data
 */
- (NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO *) data
{
  return _data;
}

/**
 * the data
 */
- (void) setData: (NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO *) newData
{
  [newData retain];
  [_data release];
  _data = newData;
}

- (void) dealloc
{
  [self setData: nil];
  [super dealloc];
}

//documentation inherited.
+ (id<EnunciateXML>) readFromXML: (NSData *) xml
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest *_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest;
  xmlTextReaderPtr reader = xmlReaderForMemory([xml bytes], [xml length], NULL, NULL, 0);
  if (reader == NULL) {
    [NSException raise: @"XMLReadError"
                 format: @"Error instantiating an XML reader."];
    return nil;
  }

  _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest = (NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest *) [NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest readXMLElement: reader];
  xmlFreeTextReader(reader); //free the reader
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest;
}

//documentation inherited.
- (NSData *) writeToXML
{
  xmlBufferPtr buf;
  xmlTextWriterPtr writer;
  int rc;
  NSData *data;

  buf = xmlBufferCreate();
  if (buf == NULL) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML buffer."];
    return nil;
  }

  writer = xmlNewTextWriterMemory(buf, 0);
  if (writer == NULL) {
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML writer."];
    return nil;
  }

  rc = xmlTextWriterStartDocument(writer, NULL, "utf-8", NULL);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML start document."];
    return nil;
  }

  NS_DURING
  {
    [self writeXMLElement: writer];
  }
  NS_HANDLER
  {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [localException raise];
  }
  NS_ENDHANDLER

  rc = xmlTextWriterEndDocument(writer);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML end document."];
    return nil;
  }

  xmlFreeTextWriter(writer);
  data = [NSData dataWithBytes: buf->content length: buf->use];
  xmlBufferFree(buf);
  return data;
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest */

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest (JAXB) <JAXBReading, JAXBWriting, JAXBType, JAXBElement>

@end /*interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest (JAXB)*/

/**
 * Internal, private implementation for JAXB reading and writing.
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest (JAXB)

/**
 * Read an instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest defined by the XML reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest *_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest = [[NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest alloc] init];
  NS_DURING
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest initWithReader: reader];
  }
  NS_HANDLER
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest dealloc];
    _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest autorelease];
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest;
}

/**
 * Initialize this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest according to
 * the XML being read from the reader.
 *
 * @param reader The reader.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  return [super initWithReader: reader];
}

/**
 * Write the XML for this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest to the writer.
 * Note that since we're only writing the XML type,
 * No start/end element will be written.
 *
 * @param reader The reader.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [super writeXMLType:writer];
}

/**
 * Reads a NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest from an XML reader. The element to be read is
 * "userAndGroupConfigTest".
 *
 * @param reader The XML reader.
 * @return The NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest.
 */
+ (id<JAXBElement>) readXMLElement: (xmlTextReaderPtr) reader {
  int status;
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest *_ldapUserAndGroupConfigTestRequest = nil;

  if (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT) {
    status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
    if (status < 1) {
      [NSException raise: @"XMLReadError"
                   format: @"Error advancing the reader to start element userAndGroupConfigTest."];
    }
  }

  if (xmlStrcmp(BAD_CAST "userAndGroupConfigTest", xmlTextReaderConstLocalName(reader)) == 0
      && xmlTextReaderConstNamespaceUri(reader) == NULL) {
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read root element {}userAndGroupConfigTest.");
#endif
    _ldapUserAndGroupConfigTestRequest = (NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest *)[NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Successfully read root element {}userAndGroupConfigTest.");
#endif
  }
  else {
    if (xmlTextReaderConstNamespaceUri(reader) == NULL) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest. Expected element userAndGroupConfigTest. Current element: {}%s", xmlTextReaderConstLocalName(reader)];
    }
    else {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest. Expected element userAndGroupConfigTest. Current element: {%s}%s\n", xmlTextReaderConstNamespaceUri(reader), xmlTextReaderConstLocalName(reader)];
    }
  }

  return _ldapUserAndGroupConfigTestRequest;
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest to XML under element name "userAndGroupConfigTest".
 * The namespace declarations for the element will be written.
 *
 * @param writer The XML writer.
 * @param _ldapUserAndGroupConfigTestRequest The LdapUserAndGroupConfigTestRequest to write.
 * @return 1 if successful, 0 otherwise.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer
{
  [self writeXMLElement: writer writeNamespaces: YES];
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest to an XML writer.
 *
 * @param writer The writer.
 * @param writeNs Whether to write the namespaces for this element to the xml writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer writeNamespaces: (BOOL) writeNs
{
  int rc = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userAndGroupConfigTest", NULL);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing start element {}userAndGroupConfigTest. XML writer status: %i\n", rc];
  }

#if DEBUG_ENUNCIATE > 1
  NSLog(@"writing type {}ldapUserAndGroupConfigTestRequest for root element {}userAndGroupConfigTest...");
#endif
  [self writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
  NSLog(@"successfully wrote type {}ldapUserAndGroupConfigTestRequest for root element {}userAndGroupConfigTest...");
#endif
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing end element {}userAndGroupConfigTest. XML writer status: %i\n", rc];
  }
}

//documentation inherited.
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  void *_child_accessor;

  if ([super readJAXBAttribute: reader]) {
    return YES;
  }

  return NO;
}

//documentation inherited.
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return [super readJAXBValue: reader];
}

//documentation inherited.
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  id __child;
  void *_child_accessor;
  int status, depth;

  if ([super readJAXBChildElement: reader]) {
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "data", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}data of type {}ldapUserAndGroupConfigTestRequestDTO.");
#endif
    __child = [NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}data of type {}ldapUserAndGroupConfigTestRequestDTO.");
#endif

    [self setData: __child];
    return YES;
  } //end "if choice"


  return NO;
}

//documentation inherited.
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return [super readUnknownJAXBChildElement: reader];
}

//documentation inherited.
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
  [super readUnknownJAXBAttribute: reader];
}

//documentation inherited.
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  int status;

  [super writeJAXBAttributes: writer];

}

//documentation inherited.
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  [super writeJAXBValue: writer];
}

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  int status;
  id __item;
  NSEnumerator *__enumerator;

  [super writeJAXBChildElements: writer];

  if ([self data]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "data", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}data."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}data...");
#endif
    [[self data] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}data...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}data."];
    }
  }
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest (JAXB) */

#endif /* DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequest_M */
#ifndef DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO_M
#define DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO_M

/**
 * (no documentation provided)
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO

/**
 * the userId
 */
- (NSString *) userId
{
  return _userId;
}

/**
 * the userId
 */
- (void) setUserId: (NSString *) newUserId
{
  [newUserId retain];
  [_userId release];
  _userId = newUserId;
}

/**
 * the roles
 */
- (NSArray *) roles
{
  return _roles;
}

/**
 * the roles
 */
- (void) setRoles: (NSArray *) newRoles
{
  [newRoles retain];
  [_roles release];
  _roles = newRoles;
}

/**
 * the name
 */
- (NSString *) name
{
  return _name;
}

/**
 * the name
 */
- (void) setName: (NSString *) newName
{
  [newName retain];
  [_name release];
  _name = newName;
}

/**
 * the email
 */
- (NSString *) email
{
  return _email;
}

/**
 * the email
 */
- (void) setEmail: (NSString *) newEmail
{
  [newEmail retain];
  [_email release];
  _email = newEmail;
}

- (void) dealloc
{
  [self setUserId: nil];
  [self setRoles: nil];
  [self setName: nil];
  [self setEmail: nil];
  [super dealloc];
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO */

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO (JAXB) <JAXBReading, JAXBWriting, JAXBType>

@end /*interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO (JAXB)*/

/**
 * Internal, private implementation for JAXB reading and writing.
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO (JAXB)

/**
 * Read an instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO defined by the XML reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO *_nEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO = [[NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO alloc] init];
  NS_DURING
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO initWithReader: reader];
  }
  NS_HANDLER
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO dealloc];
    _nEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO autorelease];
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO;
}

/**
 * Initialize this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO according to
 * the XML being read from the reader.
 *
 * @param reader The reader.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  return [super initWithReader: reader];
}

/**
 * Write the XML for this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO to the writer.
 * Note that since we're only writing the XML type,
 * No start/end element will be written.
 *
 * @param reader The reader.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [super writeXMLType:writer];
}

//documentation inherited.
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  void *_child_accessor;

  if ([super readJAXBAttribute: reader]) {
    return YES;
  }

  return NO;
}

//documentation inherited.
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return [super readJAXBValue: reader];
}

//documentation inherited.
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  id __child;
  void *_child_accessor;
  int status, depth;

  if ([super readJAXBChildElement: reader]) {
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "userId", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}userId of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}userId of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setUserId: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "roles", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}roles of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}roles of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    if ([self roles]) {
      [self setRoles: [[self roles] arrayByAddingObject: __child]];
    }
    else {
      [self setRoles: [NSArray arrayWithObject: __child]];
    }
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "name", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}name of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}name of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setName: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "email", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}email of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}email of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setEmail: __child];
    return YES;
  } //end "if choice"


  return NO;
}

//documentation inherited.
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return [super readUnknownJAXBChildElement: reader];
}

//documentation inherited.
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
  [super readUnknownJAXBAttribute: reader];
}

//documentation inherited.
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  int status;

  [super writeJAXBAttributes: writer];

}

//documentation inherited.
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  [super writeJAXBValue: writer];
}

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  int status;
  id __item;
  NSEnumerator *__enumerator;

  [super writeJAXBChildElements: writer];

  if ([self userId]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userId", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}userId."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}userId...");
#endif
    [[self userId] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}userId...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}userId."];
    }
  }
  if ([self roles]) {
    __enumerator = [[self roles] objectEnumerator];

    while ( (__item = [__enumerator nextObject]) ) {
      status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "roles", NULL);
      if (status < 0) {
        [NSException raise: @"XMLWriteError"
                     format: @"Error writing start child element {}roles."];
      }

#if DEBUG_ENUNCIATE > 1
      NSLog(@"writing element {}roles...");
#endif
      [__item writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
      NSLog(@"successfully wrote element {}roles...");
#endif

      status = xmlTextWriterEndElement(writer);
      if (status < 0) {
        [NSException raise: @"XMLWriteError"
                     format: @"Error writing end child element {}roles."];
      }
    } //end item iterator.
  }
  if ([self name]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "name", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}name."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}name...");
#endif
    [[self name] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}name...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}name."];
    }
  }
  if ([self email]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "email", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}email."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}email...");
#endif
    [[self email] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}email...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}email."];
    }
  }
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO (JAXB) */

#endif /* DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserResponseDTO_M */
#ifndef DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse_M
#define DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse_M

/**
 * (no documentation provided)
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse

/**
 * the userAndGroupConfig
 */
- (NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO *) data
{
  return _data;
}

/**
 * the userAndGroupConfig
 */
- (void) setData: (NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO *) newData
{
  [newData retain];
  [_data release];
  _data = newData;
}

- (void) dealloc
{
  [self setData: nil];
  [super dealloc];
}

//documentation inherited.
+ (id<EnunciateXML>) readFromXML: (NSData *) xml
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse *_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse;
  xmlTextReaderPtr reader = xmlReaderForMemory([xml bytes], [xml length], NULL, NULL, 0);
  if (reader == NULL) {
    [NSException raise: @"XMLReadError"
                 format: @"Error instantiating an XML reader."];
    return nil;
  }

  _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse = (NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse *) [NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse readXMLElement: reader];
  xmlFreeTextReader(reader); //free the reader
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse;
}

//documentation inherited.
- (NSData *) writeToXML
{
  xmlBufferPtr buf;
  xmlTextWriterPtr writer;
  int rc;
  NSData *data;

  buf = xmlBufferCreate();
  if (buf == NULL) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML buffer."];
    return nil;
  }

  writer = xmlNewTextWriterMemory(buf, 0);
  if (writer == NULL) {
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML writer."];
    return nil;
  }

  rc = xmlTextWriterStartDocument(writer, NULL, "utf-8", NULL);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML start document."];
    return nil;
  }

  NS_DURING
  {
    [self writeXMLElement: writer];
  }
  NS_HANDLER
  {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [localException raise];
  }
  NS_ENDHANDLER

  rc = xmlTextWriterEndDocument(writer);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML end document."];
    return nil;
  }

  xmlFreeTextWriter(writer);
  data = [NSData dataWithBytes: buf->content length: buf->use];
  xmlBufferFree(buf);
  return data;
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse */

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse (JAXB) <JAXBReading, JAXBWriting, JAXBType, JAXBElement>

@end /*interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse (JAXB)*/

/**
 * Internal, private implementation for JAXB reading and writing.
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse (JAXB)

/**
 * Read an instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse defined by the XML reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse *_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse = [[NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse alloc] init];
  NS_DURING
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse initWithReader: reader];
  }
  NS_HANDLER
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse dealloc];
    _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse autorelease];
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse;
}

/**
 * Initialize this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse according to
 * the XML being read from the reader.
 *
 * @param reader The reader.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  return [super initWithReader: reader];
}

/**
 * Write the XML for this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse to the writer.
 * Note that since we're only writing the XML type,
 * No start/end element will be written.
 *
 * @param reader The reader.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [super writeXMLType:writer];
}

/**
 * Reads a NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse from an XML reader. The element to be read is
 * "userAndGroupConfig".
 *
 * @param reader The XML reader.
 * @return The NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse.
 */
+ (id<JAXBElement>) readXMLElement: (xmlTextReaderPtr) reader {
  int status;
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse *_ldapUserAndGroupConfigurationResponse = nil;

  if (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT) {
    status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
    if (status < 1) {
      [NSException raise: @"XMLReadError"
                   format: @"Error advancing the reader to start element userAndGroupConfig."];
    }
  }

  if (xmlStrcmp(BAD_CAST "userAndGroupConfig", xmlTextReaderConstLocalName(reader)) == 0
      && xmlTextReaderConstNamespaceUri(reader) == NULL) {
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read root element {}userAndGroupConfig.");
#endif
    _ldapUserAndGroupConfigurationResponse = (NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse *)[NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Successfully read root element {}userAndGroupConfig.");
#endif
  }
  else {
    if (xmlTextReaderConstNamespaceUri(reader) == NULL) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse. Expected element userAndGroupConfig. Current element: {}%s", xmlTextReaderConstLocalName(reader)];
    }
    else {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse. Expected element userAndGroupConfig. Current element: {%s}%s\n", xmlTextReaderConstNamespaceUri(reader), xmlTextReaderConstLocalName(reader)];
    }
  }

  return _ldapUserAndGroupConfigurationResponse;
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse to XML under element name "userAndGroupConfig".
 * The namespace declarations for the element will be written.
 *
 * @param writer The XML writer.
 * @param _ldapUserAndGroupConfigurationResponse The LdapUserAndGroupConfigurationResponse to write.
 * @return 1 if successful, 0 otherwise.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer
{
  [self writeXMLElement: writer writeNamespaces: YES];
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse to an XML writer.
 *
 * @param writer The writer.
 * @param writeNs Whether to write the namespaces for this element to the xml writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer writeNamespaces: (BOOL) writeNs
{
  int rc = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userAndGroupConfig", NULL);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing start element {}userAndGroupConfig. XML writer status: %i\n", rc];
  }

#if DEBUG_ENUNCIATE > 1
  NSLog(@"writing type {}ldapUserAndGroupConfigurationResponse for root element {}userAndGroupConfig...");
#endif
  [self writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
  NSLog(@"successfully wrote type {}ldapUserAndGroupConfigurationResponse for root element {}userAndGroupConfig...");
#endif
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing end element {}userAndGroupConfig. XML writer status: %i\n", rc];
  }
}

//documentation inherited.
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  void *_child_accessor;

  if ([super readJAXBAttribute: reader]) {
    return YES;
  }

  return NO;
}

//documentation inherited.
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return [super readJAXBValue: reader];
}

//documentation inherited.
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  id __child;
  void *_child_accessor;
  int status, depth;

  if ([super readJAXBChildElement: reader]) {
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "data", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}data of type {}ldapUserAndGroupConfigurationDTO.");
#endif
    __child = [NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationDTO readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}data of type {}ldapUserAndGroupConfigurationDTO.");
#endif

    [self setData: __child];
    return YES;
  } //end "if choice"


  return NO;
}

//documentation inherited.
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return [super readUnknownJAXBChildElement: reader];
}

//documentation inherited.
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
  [super readUnknownJAXBAttribute: reader];
}

//documentation inherited.
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  int status;

  [super writeJAXBAttributes: writer];

}

//documentation inherited.
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  [super writeJAXBValue: writer];
}

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  int status;
  id __item;
  NSEnumerator *__enumerator;

  [super writeJAXBChildElements: writer];

  if ([self data]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "data", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}data."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}data...");
#endif
    [[self data] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}data...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}data."];
    }
  }
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse (JAXB) */

#endif /* DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigurationResponse_M */
#ifndef DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse_M
#define DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse_M

/**
 * (no documentation provided)
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse

/**
 * the connectionInfo
 */
- (NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO *) data
{
  return _data;
}

/**
 * the connectionInfo
 */
- (void) setData: (NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO *) newData
{
  [newData retain];
  [_data release];
  _data = newData;
}

- (void) dealloc
{
  [self setData: nil];
  [super dealloc];
}

//documentation inherited.
+ (id<EnunciateXML>) readFromXML: (NSData *) xml
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse *_nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse;
  xmlTextReaderPtr reader = xmlReaderForMemory([xml bytes], [xml length], NULL, NULL, 0);
  if (reader == NULL) {
    [NSException raise: @"XMLReadError"
                 format: @"Error instantiating an XML reader."];
    return nil;
  }

  _nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse = (NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse *) [NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse readXMLElement: reader];
  xmlFreeTextReader(reader); //free the reader
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse;
}

//documentation inherited.
- (NSData *) writeToXML
{
  xmlBufferPtr buf;
  xmlTextWriterPtr writer;
  int rc;
  NSData *data;

  buf = xmlBufferCreate();
  if (buf == NULL) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML buffer."];
    return nil;
  }

  writer = xmlNewTextWriterMemory(buf, 0);
  if (writer == NULL) {
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error creating an XML writer."];
    return nil;
  }

  rc = xmlTextWriterStartDocument(writer, NULL, "utf-8", NULL);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML start document."];
    return nil;
  }

  NS_DURING
  {
    [self writeXMLElement: writer];
  }
  NS_HANDLER
  {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [localException raise];
  }
  NS_ENDHANDLER

  rc = xmlTextWriterEndDocument(writer);
  if (rc < 0) {
    xmlFreeTextWriter(writer);
    xmlBufferFree(buf);
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing XML end document."];
    return nil;
  }

  xmlFreeTextWriter(writer);
  data = [NSData dataWithBytes: buf->content length: buf->use];
  xmlBufferFree(buf);
  return data;
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse */

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse (JAXB) <JAXBReading, JAXBWriting, JAXBType, JAXBElement>

@end /*interface NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse (JAXB)*/

/**
 * Internal, private implementation for JAXB reading and writing.
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse (JAXB)

/**
 * Read an instance of NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse defined by the XML reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse *_nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse = [[NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse alloc] init];
  NS_DURING
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse initWithReader: reader];
  }
  NS_HANDLER
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse dealloc];
    _nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [_nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse autorelease];
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse;
}

/**
 * Initialize this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse according to
 * the XML being read from the reader.
 *
 * @param reader The reader.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  return [super initWithReader: reader];
}

/**
 * Write the XML for this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse to the writer.
 * Note that since we're only writing the XML type,
 * No start/end element will be written.
 *
 * @param reader The reader.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [super writeXMLType:writer];
}

/**
 * Reads a NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse from an XML reader. The element to be read is
 * "connectionInfo".
 *
 * @param reader The XML reader.
 * @return The NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse.
 */
+ (id<JAXBElement>) readXMLElement: (xmlTextReaderPtr) reader {
  int status;
  NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse *_ldapConnectionInfoResponse = nil;

  if (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT) {
    status = xmlTextReaderAdvanceToNextStartOrEndElement(reader);
    if (status < 1) {
      [NSException raise: @"XMLReadError"
                   format: @"Error advancing the reader to start element connectionInfo."];
    }
  }

  if (xmlStrcmp(BAD_CAST "connectionInfo", xmlTextReaderConstLocalName(reader)) == 0
      && xmlTextReaderConstNamespaceUri(reader) == NULL) {
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read root element {}connectionInfo.");
#endif
    _ldapConnectionInfoResponse = (NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse *)[NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"Successfully read root element {}connectionInfo.");
#endif
  }
  else {
    if (xmlTextReaderConstNamespaceUri(reader) == NULL) {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse. Expected element connectionInfo. Current element: {}%s", xmlTextReaderConstLocalName(reader)];
    }
    else {
      [NSException raise: @"XMLReadError"
                   format: @"Unable to read NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse. Expected element connectionInfo. Current element: {%s}%s\n", xmlTextReaderConstNamespaceUri(reader), xmlTextReaderConstLocalName(reader)];
    }
  }

  return _ldapConnectionInfoResponse;
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse to XML under element name "connectionInfo".
 * The namespace declarations for the element will be written.
 *
 * @param writer The XML writer.
 * @param _ldapConnectionInfoResponse The LdapConnectionInfoResponse to write.
 * @return 1 if successful, 0 otherwise.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer
{
  [self writeXMLElement: writer writeNamespaces: YES];
}

/**
 * Writes this NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse to an XML writer.
 *
 * @param writer The writer.
 * @param writeNs Whether to write the namespaces for this element to the xml writer.
 */
- (void) writeXMLElement: (xmlTextWriterPtr) writer writeNamespaces: (BOOL) writeNs
{
  int rc = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "connectionInfo", NULL);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing start element {}connectionInfo. XML writer status: %i\n", rc];
  }

#if DEBUG_ENUNCIATE > 1
  NSLog(@"writing type {}ldapConnectionInfoResponse for root element {}connectionInfo...");
#endif
  [self writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
  NSLog(@"successfully wrote type {}ldapConnectionInfoResponse for root element {}connectionInfo...");
#endif
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) {
    [NSException raise: @"XMLWriteError"
                 format: @"Error writing end element {}connectionInfo. XML writer status: %i\n", rc];
  }
}

//documentation inherited.
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  void *_child_accessor;

  if ([super readJAXBAttribute: reader]) {
    return YES;
  }

  return NO;
}

//documentation inherited.
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return [super readJAXBValue: reader];
}

//documentation inherited.
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  id __child;
  void *_child_accessor;
  int status, depth;

  if ([super readJAXBChildElement: reader]) {
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "data", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}data of type {}ldapConnectionInfoDTO.");
#endif
    __child = [NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}data of type {}ldapConnectionInfoDTO.");
#endif

    [self setData: __child];
    return YES;
  } //end "if choice"


  return NO;
}

//documentation inherited.
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return [super readUnknownJAXBChildElement: reader];
}

//documentation inherited.
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
  [super readUnknownJAXBAttribute: reader];
}

//documentation inherited.
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  int status;

  [super writeJAXBAttributes: writer];

}

//documentation inherited.
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  [super writeJAXBValue: writer];
}

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  int status;
  id __item;
  NSEnumerator *__enumerator;

  [super writeJAXBChildElements: writer];

  if ([self data]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "data", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}data."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}data...");
#endif
    [[self data] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}data...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}data."];
    }
  }
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse (JAXB) */

#endif /* DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoResponse_M */
#ifndef DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO_M
#define DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO_M

/**
 * (no documentation provided)
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO

/**
 * the searchBase
 */
- (NSString *) searchBase
{
  return _searchBase;
}

/**
 * the searchBase
 */
- (void) setSearchBase: (NSString *) newSearchBase
{
  [newSearchBase retain];
  [_searchBase release];
  _searchBase = newSearchBase;
}

/**
 * the systemUsername
 */
- (NSString *) systemUsername
{
  return _systemUsername;
}

/**
 * the systemUsername
 */
- (void) setSystemUsername: (NSString *) newSystemUsername
{
  [newSystemUsername retain];
  [_systemUsername release];
  _systemUsername = newSystemUsername;
}

/**
 * the systemPassword
 */
- (NSString *) systemPassword
{
  return _systemPassword;
}

/**
 * the systemPassword
 */
- (void) setSystemPassword: (NSString *) newSystemPassword
{
  [newSystemPassword retain];
  [_systemPassword release];
  _systemPassword = newSystemPassword;
}

/**
 * the authScheme
 */
- (NSString *) authScheme
{
  return _authScheme;
}

/**
 * the authScheme
 */
- (void) setAuthScheme: (NSString *) newAuthScheme
{
  [newAuthScheme retain];
  [_authScheme release];
  _authScheme = newAuthScheme;
}

/**
 * the protocol
 */
- (NSString *) protocol
{
  return _protocol;
}

/**
 * the protocol
 */
- (void) setProtocol: (NSString *) newProtocol
{
  [newProtocol retain];
  [_protocol release];
  _protocol = newProtocol;
}

/**
 * the host
 */
- (NSString *) host
{
  return _host;
}

/**
 * the host
 */
- (void) setHost: (NSString *) newHost
{
  [newHost retain];
  [_host release];
  _host = newHost;
}

/**
 * the port
 */
- (int) port
{
  return _port;
}

/**
 * the port
 */
- (void) setPort: (int) newPort
{
  _port = newPort;
}

/**
 * the realm
 */
- (NSString *) realm
{
  return _realm;
}

/**
 * the realm
 */
- (void) setRealm: (NSString *) newRealm
{
  [newRealm retain];
  [_realm release];
  _realm = newRealm;
}

- (void) dealloc
{
  [self setSearchBase: nil];
  [self setSystemUsername: nil];
  [self setSystemPassword: nil];
  [self setAuthScheme: nil];
  [self setProtocol: nil];
  [self setHost: nil];
  [self setRealm: nil];
  [super dealloc];
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO */

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO (JAXB) <JAXBReading, JAXBWriting, JAXBType>

@end /*interface NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO (JAXB)*/

/**
 * Internal, private implementation for JAXB reading and writing.
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO (JAXB)

/**
 * Read an instance of NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO defined by the XML reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO *_nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO = [[NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO alloc] init];
  NS_DURING
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO initWithReader: reader];
  }
  NS_HANDLER
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO dealloc];
    _nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [_nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO autorelease];
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO;
}

/**
 * Initialize this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO according to
 * the XML being read from the reader.
 *
 * @param reader The reader.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  return [super initWithReader: reader];
}

/**
 * Write the XML for this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO to the writer.
 * Note that since we're only writing the XML type,
 * No start/end element will be written.
 *
 * @param reader The reader.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [super writeXMLType:writer];
}

//documentation inherited.
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  void *_child_accessor;

  if ([super readJAXBAttribute: reader]) {
    return YES;
  }

  return NO;
}

//documentation inherited.
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return [super readJAXBValue: reader];
}

//documentation inherited.
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  id __child;
  void *_child_accessor;
  int status, depth;

  if ([super readJAXBChildElement: reader]) {
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "searchBase", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}searchBase of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}searchBase of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setSearchBase: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "systemUsername", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}systemUsername of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}systemUsername of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setSystemUsername: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "systemPassword", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}systemPassword of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}systemPassword of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setSystemPassword: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "authScheme", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}authScheme of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}authScheme of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setAuthScheme: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "protocol", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}protocol of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}protocol of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setProtocol: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "host", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}host of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}host of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setHost: __child];
    return YES;
  } //end "if choice"


  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "port", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

    _child_accessor = xmlTextReaderReadIntType(reader);
    if (_child_accessor == NULL) {
      //panic: unable to return the value for some reason.
      [self dealloc];
      [NSException raise: @"XMLReadError"
                   format: @"Error reading element value."];
    }
    [self setPort: *((int*) _child_accessor)];
    free(_child_accessor);
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "realm", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}realm of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}realm of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setRealm: __child];
    return YES;
  } //end "if choice"


  return NO;
}

//documentation inherited.
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return [super readUnknownJAXBChildElement: reader];
}

//documentation inherited.
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
  [super readUnknownJAXBAttribute: reader];
}

//documentation inherited.
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  int status;

  [super writeJAXBAttributes: writer];

}

//documentation inherited.
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  [super writeJAXBValue: writer];
}

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  int status;
  id __item;
  NSEnumerator *__enumerator;

  [super writeJAXBChildElements: writer];

  if ([self searchBase]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "searchBase", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}searchBase."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}searchBase...");
#endif
    [[self searchBase] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}searchBase...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}searchBase."];
    }
  }
  if ([self systemUsername]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "systemUsername", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}systemUsername."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}systemUsername...");
#endif
    [[self systemUsername] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}systemUsername...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}systemUsername."];
    }
  }
  if ([self systemPassword]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "systemPassword", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}systemPassword."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}systemPassword...");
#endif
    [[self systemPassword] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}systemPassword...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}systemPassword."];
    }
  }
  if ([self authScheme]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "authScheme", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}authScheme."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}authScheme...");
#endif
    [[self authScheme] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}authScheme...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}authScheme."];
    }
  }
  if ([self protocol]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "protocol", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}protocol."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}protocol...");
#endif
    [[self protocol] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}protocol...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}protocol."];
    }
  }
  if ([self host]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "host", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}host."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}host...");
#endif
    [[self host] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}host...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}host."];
    }
  }
  if (YES) { //always write the primitive element...
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "port", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}port."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}port...");
#endif
    status = xmlTextWriterWriteIntType(writer, &_port);
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}port...");
#endif
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing child element {}port."];
    }

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}port."];
    }
  }
  if ([self realm]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "realm", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}realm."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}realm...");
#endif
    [[self realm] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}realm...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}realm."];
    }
  }
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO (JAXB) */

#endif /* DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapConnectionInfoDTO_M */
#ifndef DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO_M
#define DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO_M

/**
 * (no documentation provided)
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO

/**
 * the searchBase
 */
- (NSString *) searchBase
{
  return _searchBase;
}

/**
 * the searchBase
 */
- (void) setSearchBase: (NSString *) newSearchBase
{
  [newSearchBase retain];
  [_searchBase release];
  _searchBase = newSearchBase;
}

/**
 * the systemUsername
 */
- (NSString *) systemUsername
{
  return _systemUsername;
}

/**
 * the systemUsername
 */
- (void) setSystemUsername: (NSString *) newSystemUsername
{
  [newSystemUsername retain];
  [_systemUsername release];
  _systemUsername = newSystemUsername;
}

/**
 * the systemPassword
 */
- (NSString *) systemPassword
{
  return _systemPassword;
}

/**
 * the systemPassword
 */
- (void) setSystemPassword: (NSString *) newSystemPassword
{
  [newSystemPassword retain];
  [_systemPassword release];
  _systemPassword = newSystemPassword;
}

/**
 * the authScheme
 */
- (NSString *) authScheme
{
  return _authScheme;
}

/**
 * the authScheme
 */
- (void) setAuthScheme: (NSString *) newAuthScheme
{
  [newAuthScheme retain];
  [_authScheme release];
  _authScheme = newAuthScheme;
}

/**
 * the protocol
 */
- (NSString *) protocol
{
  return _protocol;
}

/**
 * the protocol
 */
- (void) setProtocol: (NSString *) newProtocol
{
  [newProtocol retain];
  [_protocol release];
  _protocol = newProtocol;
}

/**
 * the host
 */
- (NSString *) host
{
  return _host;
}

/**
 * the host
 */
- (void) setHost: (NSString *) newHost
{
  [newHost retain];
  [_host release];
  _host = newHost;
}

/**
 * the port
 */
- (int) port
{
  return _port;
}

/**
 * the port
 */
- (void) setPort: (int) newPort
{
  _port = newPort;
}

/**
 * the realm
 */
- (NSString *) realm
{
  return _realm;
}

/**
 * the realm
 */
- (void) setRealm: (NSString *) newRealm
{
  [newRealm retain];
  [_realm release];
  _realm = newRealm;
}

/**
 * (no documentation provided)
 */
- (int) userLimitCount
{
  return _userLimitCount;
}

/**
 * (no documentation provided)
 */
- (void) setUserLimitCount: (int) newUserLimitCount
{
  _userLimitCount = newUserLimitCount;
}

- (void) dealloc
{
  [self setSearchBase: nil];
  [self setSystemUsername: nil];
  [self setSystemPassword: nil];
  [self setAuthScheme: nil];
  [self setProtocol: nil];
  [self setHost: nil];
  [self setRealm: nil];
  [super dealloc];
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO */

/**
 * Internal, private interface for JAXB reading and writing.
 */
@interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO (JAXB) <JAXBReading, JAXBWriting, JAXBType>

@end /*interface NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO (JAXB)*/

/**
 * Internal, private implementation for JAXB reading and writing.
 */
@implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO (JAXB)

/**
 * Read an instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO from an XML reader.
 *
 * @param reader The reader.
 * @return An instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO defined by the XML reader.
 */
+ (id<JAXBType>) readXMLType: (xmlTextReaderPtr) reader
{
  NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO *_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO = [[NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO alloc] init];
  NS_DURING
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO initWithReader: reader];
  }
  NS_HANDLER
  {
    [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO dealloc];
    _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO = nil;
    [localException raise];
  }
  NS_ENDHANDLER

  [_nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO autorelease];
  return _nEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO;
}

/**
 * Initialize this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO according to
 * the XML being read from the reader.
 *
 * @param reader The reader.
 */
- (id) initWithReader: (xmlTextReaderPtr) reader
{
  return [super initWithReader: reader];
}

/**
 * Write the XML for this instance of NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO to the writer.
 * Note that since we're only writing the XML type,
 * No start/end element will be written.
 *
 * @param reader The reader.
 */
- (void) writeXMLType: (xmlTextWriterPtr) writer
{
  [super writeXMLType:writer];
}

//documentation inherited.
- (BOOL) readJAXBAttribute: (xmlTextReaderPtr) reader
{
  void *_child_accessor;

  if ([super readJAXBAttribute: reader]) {
    return YES;
  }

  return NO;
}

//documentation inherited.
- (BOOL) readJAXBValue: (xmlTextReaderPtr) reader
{
  return [super readJAXBValue: reader];
}

//documentation inherited.
- (BOOL) readJAXBChildElement: (xmlTextReaderPtr) reader
{
  id __child;
  void *_child_accessor;
  int status, depth;

  if ([super readJAXBChildElement: reader]) {
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "searchBase", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}searchBase of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}searchBase of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setSearchBase: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "systemUsername", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}systemUsername of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}systemUsername of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setSystemUsername: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "systemPassword", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}systemPassword of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}systemPassword of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setSystemPassword: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "authScheme", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}authScheme of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}authScheme of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setAuthScheme: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "protocol", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}protocol of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}protocol of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setProtocol: __child];
    return YES;
  } //end "if choice"

  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "host", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}host of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}host of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setHost: __child];
    return YES;
  } //end "if choice"


  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "port", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

    _child_accessor = xmlTextReaderReadIntType(reader);
    if (_child_accessor == NULL) {
      //panic: unable to return the value for some reason.
      [self dealloc];
      [NSException raise: @"XMLReadError"
                   format: @"Error reading element value."];
    }
    [self setPort: *((int*) _child_accessor)];
    free(_child_accessor);
    return YES;
  }
  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "realm", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

#if DEBUG_ENUNCIATE > 1
    NSLog(@"Attempting to read choice {}realm of type {http://www.w3.org/2001/XMLSchema}string.");
#endif
    __child = [NSString readXMLType: reader];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully read choice {}realm of type {http://www.w3.org/2001/XMLSchema}string.");
#endif

    [self setRealm: __child];
    return YES;
  } //end "if choice"


  if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
    && xmlStrcmp(BAD_CAST "userLimitCount", xmlTextReaderConstLocalName(reader)) == 0
    && xmlTextReaderConstNamespaceUri(reader) == NULL) {

    _child_accessor = xmlTextReaderReadIntType(reader);
    if (_child_accessor == NULL) {
      //panic: unable to return the value for some reason.
      [self dealloc];
      [NSException raise: @"XMLReadError"
                   format: @"Error reading element value."];
    }
    [self setUserLimitCount: *((int*) _child_accessor)];
    free(_child_accessor);
    return YES;
  }

  return NO;
}

//documentation inherited.
- (int) readUnknownJAXBChildElement: (xmlTextReaderPtr) reader
{
  return [super readUnknownJAXBChildElement: reader];
}

//documentation inherited.
- (void) readUnknownJAXBAttribute: (xmlTextReaderPtr) reader
{
  [super readUnknownJAXBAttribute: reader];
}

//documentation inherited.
- (void) writeJAXBAttributes: (xmlTextWriterPtr) writer
{
  int status;

  [super writeJAXBAttributes: writer];

}

//documentation inherited.
- (void) writeJAXBValue: (xmlTextWriterPtr) writer
{
  [super writeJAXBValue: writer];
}

/**
 * Method for writing the child elements.
 *
 * @param writer The writer.
 */
- (void) writeJAXBChildElements: (xmlTextWriterPtr) writer
{
  int status;
  id __item;
  NSEnumerator *__enumerator;

  [super writeJAXBChildElements: writer];

  if ([self searchBase]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "searchBase", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}searchBase."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}searchBase...");
#endif
    [[self searchBase] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}searchBase...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}searchBase."];
    }
  }
  if ([self systemUsername]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "systemUsername", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}systemUsername."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}systemUsername...");
#endif
    [[self systemUsername] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}systemUsername...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}systemUsername."];
    }
  }
  if ([self systemPassword]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "systemPassword", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}systemPassword."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}systemPassword...");
#endif
    [[self systemPassword] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}systemPassword...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}systemPassword."];
    }
  }
  if ([self authScheme]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "authScheme", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}authScheme."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}authScheme...");
#endif
    [[self authScheme] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}authScheme...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}authScheme."];
    }
  }
  if ([self protocol]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "protocol", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}protocol."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}protocol...");
#endif
    [[self protocol] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}protocol...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}protocol."];
    }
  }
  if ([self host]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "host", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}host."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}host...");
#endif
    [[self host] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}host...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}host."];
    }
  }
  if (YES) { //always write the primitive element...
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "port", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}port."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}port...");
#endif
    status = xmlTextWriterWriteIntType(writer, &_port);
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}port...");
#endif
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing child element {}port."];
    }

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}port."];
    }
  }
  if ([self realm]) {
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "realm", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}realm."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}realm...");
#endif
    [[self realm] writeXMLType: writer];
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}realm...");
#endif

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}realm."];
    }
  }
  if (YES) { //always write the primitive element...
    status = xmlTextWriterStartElementNS(writer, NULL, BAD_CAST "userLimitCount", NULL);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing start child element {}userLimitCount."];
    }

#if DEBUG_ENUNCIATE > 1
    NSLog(@"writing element {}userLimitCount...");
#endif
    status = xmlTextWriterWriteIntType(writer, &_userLimitCount);
#if DEBUG_ENUNCIATE > 1
    NSLog(@"successfully wrote element {}userLimitCount...");
#endif
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing child element {}userLimitCount."];
    }

    status = xmlTextWriterEndElement(writer);
    if (status < 0) {
      [NSException raise: @"XMLWriteError"
                   format: @"Error writing end child element {}userLimitCount."];
    }
  }
}
@end /* implementation NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO (JAXB) */

#endif /* DEF_NEXUS_LDAP_REALM_PLUGINNS0LdapUserAndGroupConfigTestRequestDTO_M */
